* gp20.asm

* Standalone Controller for an IDE CD-ROM Drive
* Rev 1.1 

* Copyright 1999, 2001  Jed Margolin (comments@jmargolin.com)
*   1. Permission is granted to individuals to make copies 
*      for their own use.
*
*      All other use requires permission of the copyright holder.
*
*   2. The author is not held liable for the use of the information
*      contained in the article and accompanying documentation.
*
*   3. The article and accompanying documentation may be freely 
*      distributed as long as:
*
*      a. Jed Margolin is given credit as the author.
*
*      b. The article and/or accompanying documentation may not be 
*         changed in any way without the copyright holder's consent.
*
*      c. No money is charged for distributing the article and/or 
*         accompanying documentation.
*
*      d. All those receiving or distributing the article and/or
*         accompanying documentation agree to be bound by these 
*         terms (1 - 3). 
*===========================================================================
* This version supports reading the switches during Timer Interrupts.
*
* All functions that use the Ports outside the Timer Interrupt must 
*  disable interrupts before they use the port(s) and then re-enable 
*  interrupts when they are done. 
*	SEI			; disable interrupts
*	CLI			; re-enable interrupts
*===========================================================================
#pagelength 65t
#pagewidth 124t
$base 10t

$INCLUDE 'gp20.h'      ; Equates for GP20
$INCLUDE 'gp20ram.h'   ; Ram Variables
$INCLUDE 'gp20jmp.h'   ; JUMP Table for PC Development Mode
*===========================================================================
****************************************************************************
* Program Initialization
* Initialize the Stack, the PLL, the I/O Ports, and the Serial Port.
****************************************************************************
*===========================================================================
* Now the Program
* First initialize any control registers and variables
START:  MOV	#$01,CONFIG1	; disable COP
	MOV	#$01,CONFIG2	; SCI from bus clock

	LDHX	#$00FF	; locate the stack
	TXS
*=============================================================================
* Set up PLL, using a 32.768 Xtal
* fBus = 4.9152 MHz	   fBus = 4.0 MHz	   fBus = 8.0 MHz
*      R = 1			R = 1			R = 1
*      N = $0258		N = $01E9		N = $03D1 
*      P = 0			P = 0			P = 0 
*      E = 2 			E = 1			E = 2
*      L = $80			L = $D1			L = $D0
*-------------------
* PCTL - PLL Control ->		   |  P  |     |  E  |
*	   7     6      5     4    3     2     1     0
*        PLLIE  PLLF  PLLON  BCS  PRE1  PRE0  VPR1  VPR0	
*Reset     0     0      1     0    0     0     0     0		
* 4MHz     0     0      0     0    0     0     0     1		$01
* 4.9MHz   0     0      0     0    0     0     1     0		$02
* 8MHz     0     0      0     0    0     0     1     0		$02
	MOV	#$02,PCTL	; 4.9152 MHz
*-------------------
* PBWC - PLL Bandwidth Control (Leave Auto off)
*          7     6      5     4    3     2     1     0
*        AUTO  LOCK   /ACQ   
*Reset     0     0      0     0    0     0     0     0		
* 4/8MHz   0     0      0     0    0     0     0     0		$00
	MOV	#$00,PBWC	; 4/8MHz
*-------------------
* PMSH - PLL Multiplier Select High (PLL must be off)  ->N High
*          7     6      5     4    3     2     1     0
*                                  M11  M10   M9    M8 
*Reset     0     0      0     0    0     0     0     0		$00
* 4MHz     0     0      0     0    0     0     0     1		$01    ($01E9)
* 4.9MHz   0     0      0     0    0     0     1     0		$01    ($0258)
* 8MHz     0     0      0     0    0     0     1     1		$03    ($03D1)
	MOV	#$02,PMSH	; 4.9152 MHz
*-------------------
* PMSL - PLL Multiplier Select Low (PLL must be off)  ->N Low
*          7     6      5     4    3     2     1     0
*          M7   M6     M5    M4   M3    M2    M1    M0
*Reset     0     0      0     0    0     0     0     0		$20
* 4MHz     1     1      1     0    1     0     0     1		$E9    ($01E9)
* 4.9MHz   0     1      0     1    1     0     0     0		$E9    ($0258)
* 8MHz     1     1      0     1    0     0     0     1		$D1    ($03D1)
	MOV	#$58,PMSL	; 4.9152 MHz
*-------------------
* PMRS - PLL VCO Range Select (PLL must be off)  ->L
*          7     6      5     4    3     2     1     0
*Reset     0     1      0     0    0     0     0     0		$20
* 4MHz     1     1      0     1    0     0     0     1		$D1
* 4.9MHz   1     0      0     0    0     0     0     0		$80
* 8MHz     1     1      0     1    0     0     0     0		$D0
	MOV	#$80,PMRS	; 4.9152 MHz
*-------------------
* PMDS - PLL Reference Divider Select  ->R
*          7     6      5     4    3     2     1     0
*                                 RDS3  RDS2  RDS1  RDS0
*Reset     0     0      0     0    0     0     0     1		
* 4/8MHz   0     0      0     0    0     0     0     1		$01
	MOV	#$01,PMDS	; 4/8MHz
*-------------------
* At this point, with a 32.768 KHz xtal, the Bus clock is 8.192 KHz = 122us 
* Turn on PLL
	BSET	PLLON,PCTL

* Wait a little
	LDX	#10
mbr1:	DBNZX	mbr1

* Select the VCO Clock
	BSET	BCS,PCTL
*=============================================================================
* Initialize I/O Ports
	CLR	DDRA	; Inputs
	CLR	DDRB	; Inputs

	LDA	#$01	; d0 = TXOUT, d1 = RXIN	(for a software UART)
	STA	PORTE
	STA	DDRE

	LDA	#$1F	; Outputs
	STA	PORTC
	STA	DDRC

	MOV	#$FF,PDPUP	; Enable Port D Pullups 

	MOV	#$10,PORTD	; make sure LATCH is high and LCDEN is low
	MOV	#$30,DDRD

* Initialize the SCI
	MOV	#$02,SCBR	; baud rate register 19.2 KB with 4.9152 Bus
	BSET	ENSCI,SCC1	; enable SCI, SCI Configuration Reg 1
	BSET	TE,SCC2		; transmit enable, SCI Configuration Reg 2
	BSET	RE,SCC2		; receive enable, SCI Configuration Reg 2
*=============================================================================
****************************************************************************
* Program Startup
* Look at the Serial Port Mode Switch and run either the Standalone CD-ROM 
*   Controller (Mode 1) or the Serial Command Function (Mode 2). Then give
*   the User the opportunity to get the Drive Info before starting the
*   CD Player. 
****************************************************************************
*===========================================================================
* If the Serial Port Mode Switch is low, do the Serial Command Program.
* Otherwise play the cd
mbr2:	BCLR	SWEN,PORTC	; select switch inputs
	BRSET	PB6,PORTA,mbr3  ; Port A inputs
	BSET	SWEN,PORTC	; deselect switch inputs

	JSR	pwr_lcd		; power-reset the lcd
	JSR	GREET_MSGS	; greeting message

        LDHX    #MSG9 		; 'Serial Port Mode'
        JSR     DOMSG

	JMP	CMD		; Serial Port Control Mode
*----------------------------
* Serial Command Mode not selected
*
* Determine whether to show the drive information 
* before going to the CD Player
mbr3:	BSET	SWEN,PORTC	; deselect switches

	JSR	pwr_lcd		; power-reset the lcd

	JSR	GREET_MSGS	; greeting message

	JSR	INIT_SWITCHES	; initialize switch variables

	JSR	Init_TIRQ	; initialize User Vector Table

	LDA	#$80		; clear the lcd display line
        JSR     CLR_DISP

	LDA	#$C0		; clear the lcd display line
        JSR     CLR_DISP

	LDHX	#IMSG0		; 'Press INFO For'
	JSR     DOMSG		; Put out the message

	LDHX	#IMSG1		; 'Drive Info'
	JSR     DOMSG		; Put out the message

* Initialize Timer 1 Interrupts
* 1ms = $09 80  at 4.912 MHz
* 4 ms =$26 00  at 4.912 MHz
	LDA	#$26
	STA	T1MODH		; must write to high byte first

	LDA	#$00
	STA	T1MODL

	LDA	#$01		; PS2, PS1, PS0; TSTOP = 0
	STA	T1SC

	BSET	TOIE,T1SC	; enable Timer 1 interrupts

	CLI			; enable interrupts

* Do a hardware reset of the CDROM Drive.
* This should be the only place that does a hardware reset.
* Note that if the tray is open, a hardware reset will close it.
* This can be disconcerting if it is not expected.
	JSR	Hard_Reset

* Give the user 4 seconds to select Drive Information
	LDA	#3	
	STA	counter

mbr4:	JSR	Wait1s

	LDA	counter
        LDX     #$D2   		; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	BRCLR	0,sw6+1,mbr5	; switch pressed

* not pressed
	DEC	counter
	BPL	mbr4

* we timed out with no switch press
* run the CD player program
	JSR	WAIT1S
	JMP	PLAY_CD

* Show the drive info, then run the CD Player
mbr5:	JSR	WAIT1S
	JSR	Drive_Info		

	JSR	WAIT1S
	JSR	WAIT1S

	JMP	PLAY_CD
*====================================================
IMSG0:  FCB  $82,'Press INFO For',$FF
IMSG1:  FCB  $C4,'Drive Info',$FF
IMSG2:  FCB  $84,'Info',$FF
IMSG3:  FCB  $C4,'CD Player',$FF
IMSG4:  FCB  $83,'Program Halted ',$FF
*=============================================================================
****************************************************************************
*  CD Player
****************************************************************************
*=============================================================================
PLAY_CD:

* Software Reset
	JSR	Soft_Reset

*  Put the address of the Main Timer Overflow function into the Vector 
*  Indirection Table.
	SEI
	JSR	Init_TIRQ
	SEI

	JSR	INIT_SWITCHES

* Set up the Timer Interrupt 
	MOV	#$26,T1MODH	; must write to high byte first
	MOV	#$00,T1MODL

	MOV	#$01,T1SC	; PS2, PS1, PS0; TSTOP = 0

	BSET	TOIE,T1SC	; enable Timer interrupts

	CLI			; Enable Interrupts

* Clear the display
	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

* Wait_For_Ready
mcdbr1:	JSR	Wait_For_Ready
	BEQ	mcdbr2
	JMP	FLASH_ERR

* Start the Disc
mcdbr2:	JSR	Start_Disc
	BEQ	mcdbr3
	JMP	FLASH_ERR

* Wait_For_Ready
mcdbr3:	JSR	Wait_For_Ready
	BEQ	mcdbr4
	JMP	FLASH_ERR

mcdbr4:	JSR	Make_TOC10	; make the Table of Contents

* Clear the display
	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

	JSR	Time_Header		; Display Header for SubCode time

* Display the number of tracks on the disc
	LDA	track_count
	JSR	BIN_BCD_99	; convert to BCD

	LDA	bcd0
        LDX     #$D2    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

* Initialize the track parameters
	LDA	#$01
	STA	track
	STA	newtrack

* Read the SubCode Time and Track
	JSR	Read_SubCode		; Track is in buffer+6
	LDA	buffer+6
	STA	track
	CLRH

* Display it
	JSR	Show_Time

	CLR	playmode
	CLR	chtrack

	LDA	#$01
	STA	track
	STA	newtrack
*------------------------------------------------------------------------
* CD Player Command Function
CD_Command:

cdlp:	

* if status(d0) [check] is set it probably means the tray has been opened
	JSR	Get_Alt_Status
	BRCLR	0,status,cdbr1

	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

	JSR	Wait_For_Ready		; wait for disc to be installed

	JSR	Soft_Reset		; software reset

	JSR	Start_Disc		; start the disc

	JSR	Make_TOC10		; Make TOC Table

	JSR	Time_Header		; Display Header for SubCode time

	LDA	track_count	; Display the number of tracks on the disc
	JSR	BIN_BCD_99	; convert to BCD
	LDA	bcd0
        LDX     #$D2    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	#$01
	STA	track
	STA	newtrack
	CLR	chtrack
	JSR	Play_Track
*---------------------------
* if sw5 and sw6 are both pressed, just stop, so we can reset it 
* without screwing up the TOC Table
* In swx+1, d0 is current debounced state. Pressed = 0
cdbr1:	BRSET	0,sw5+1,cdbr2
	BRSET	0,sw6+1,cdbr2

	BCLR	2,sw5+1
	BCLR	2,sw6+1

	SEI			; disable interrupts

	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

	LDHX	#IMSG4		; 'Program Halted'
	JSR     DOMSG

spin:	BRA	spin
*---------------------------
* Stop Switch
cdbr2:	BRCLR	2,sw2+1,cdbr3		; no
	BCLR	2,sw2+1			; yes, clear the flag

	CLR	playmode

	JSR	Do_STOP

	BRA	cdlp
*---------------------------
* Play Switch
cdbr3:	BRCLR	2,sw1+1,cdbr4		; no
	BCLR	2,sw1+1			; yes, clear the flag

* If it is already playing, don't start it
	BRSET	7,playmode,cdbr4	; continue

* Otherwise, start the track
	LDA	newtrack
	JSR	Play_Track

	BCLR	7,playmode

	BRA	cdlp
*---------------------------
* Read the SubCode Time and Track
cdbr4: 	JSR	Read_SubCode		; Track is in buffer+6
	LDA	buffer+6
	STA	track

	CLRH

* Display it
	JSR	Show_Time
*---------------------------
* Swtches are in the Interrupt routine
* Do we have a new track?
	BRSET	7,chtrack,cdbr5		; yes

* If we don't have a change of track, the disc may have played past the
* currently selected track, so update the newtrack
	LDA	track
	STA	newtrack
	JSR	BIN_BCD_99	; convert to BCD

	LDA	bcd0
        LDX     #$92    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	JMP	cdlp

* The track has been changed, so update the track and play it
cdbr5:	BCLR	7,chtrack	; clear it

	LDA	newtrack
	STA	track
	JSR	BIN_BCD_99

	LDA	bcd0
        LDX     #$92    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	newtrack
	JSR	Play_Track

	JMP	cdlp
*=============================================================================
****************************************************************************
*  Serial Port Functions
****************************************************************************
*=============================================================================
* Send byte in ACC out to the Serial Port
SENDCHAR:
* wait if a transmission is already in progress
	BRCLR	TC,SCS1,SENDCHAR 

*	LDX	SCS1   ; read SCI Status Reg 1 to clear SCI transmit empty bit
	STA	SCDR   ; send it
	RTS
*----------------------------
* Wait for, and receive a byte from, the Serial Port
* Returns with byte in ACC
GETCHAR:
	BRCLR	SCRF,SCS1,GETCHAR	; wait for character to be received
*					  If it is set, reading clears it
	
	LDA	SCDR   			; get the data
	RTS
*=============================================================================
****************************************************************************
*  Software Timing Functions
****************************************************************************
* xval = (time*4.0 -17)/3		(4.0 MHz)
* xval = (100*4.9152 - 17)/3 = 158	(FBus = 4.9152 -> 203 ns)
* time = (158*3 +17)/4.9152 = 99.89 us
WAIT100us:
        STX     savex	;		3

	LDX	#158	; 		3
wt100	DBNZX	wt100   ;		3

	LDX	savex   ;		3
	RTS		;		4
*--------------------------------------------------------------------------*
WAIT200us:
        BSR	WAIT100us
        BSR	WAIT100us
	RTS
*---------------------------------------------------------
* xval = (time*4.9152 -17)/8
* for 1000 us,  xval = (1000*4 -17)/8 = 497      (4.0 MHz)
* for 1000 us,  xval = (1000*4.9152 -17)/8 =612    (4.9152 MHz)
WAIT1ms:
        STHX    savex	;		4

	LDHX	#612	; 		3

wt1ms	AIX	#-1	;		2
	CPHX	#0000	;		3
	BNE	wt1ms   ;		3

	LDHX	savex   ;		4
	RTS		;		4
*---------------------------------------------------------
WAIT2ms:
        BSR     WAIT1ms
        BSR     WAIT1ms
        RTS
*---------------------------------------------------------
* xval = (time*4.9152 -17)/8
* for 4000 us,  xval = (4000*4 -17)/8 = 1998       (4.0 MHz)
* for 4000 us,  xval = (4000*4.9152 -17)/8 = 2455  (4.9152 MHz)
WAIT4ms:
        STHX    savex	;		4

	LDHX	#2455	; 		3

wt4ms	AIX	#-1	;		2
	CPHX	#0000	;		3
	BNE	wt4ms   ;		3

	LDHX	savex   ;		4
	RTS		;		4
*---------------------------------------------------------
WAIT10ms:
* xval = (time*4 -17)/8
* for 10000 us,  xval = (10000*4 -17)/8 = 4997	    (4.0 MHz)
* for 10000 us,  xval = (10000*4.9152 -17)/8 = 6141   (4.9152 MHz)
        STHX     savex	;		3

	LDHX	#6141	; 		3

wt10ms	AIX	#-1	;		2
	CPHX	#0000	;		3
	BNE	wt10ms  ;		3

	LDHX	savex   ;		3
	RTS		;		4
*---------------------------------------------------------
WAIT100ms:
        STA     saveacc

        LDA     #100
wt100lp: 
        BSR     WAIT1ms
        DECA
        BNE     wt100lp

        LDA     saveacc

        RTS
*---------------------------------------------------------
WAIT250ms:
        STA     saveacc

        LDA     #250
wt250lp: 
        BSR     WAIT1ms
        DECA
        BNE     wt250lp

        LDA     saveacc

        RTS
*---------------------------------------------------------
* Wait 1 second
WAIT1s:
        LDA     #100
wt1s:   BSR     WAIT10ms
        DECA
        BNE     wt1s
wt1st:  RTS
*==========================================================================
****************************************************************************
* Programs for LCD
****************************************************************************
*==========================================================================
* Power reset the lcd for 8-bit interface
* Comes up in 4-bit mode
pwr_lcd:
        JSR     WAIT10ms        ; wait 10 ms    <-----
        JSR     WAIT10ms        ; wait 10 ms    <-----
*--
        LDA     #$38            ; 8bit interface, 2 lines, 5x7 dots
        BSR     lcd_pwr_instr   ; write instruction (power-up)
        JSR     WAIT10ms        ; wait 10 ms    <-----

        LDA     #$38            ; 8bit interface, 2 lines, 5x7 dots
        BSR     lcd_pwr_instr   ; write instruction (power-up)
        JSR     WAIT200us       ; <-----

        LDA     #$38            ; 8bit interface, 2 lines, 5x7 dots
        BSR     lcd_pwr_instr   ; write instruction (power-up)
        JSR     WAIT200us       ; <-----

        LDA     #$38            ; 8bit interface, 2 lines, 5x7 dots
        BSR     lcd_pwr_instr   ; write instruction (power-up)
        JSR     WAIT200us       ; <-----
*--
        LDA     #$08            ; display off
        BSR     lcd_pwr_instr   ; write instruction (power-up)
        JSR     WAIT200us       ; <-----
*--
        LDA     #$01            ; display on
        BSR     lcd_pwr_instr   ; write instruction (power-up)
        JSR     WAIT10ms        ; wait 10 ms    <-----
*--
*       LDA     #$0E            ; display on, cursor on, blink off
        LDA     #$0F            ; display on, cursor on, blink on
*       LDA     #$0C            ; display on, cursor off, blink off
        BSR     lcd_pwr_instr   ; write instruction (power-up)
        JSR     WAIT200us       ; <-----
*--
        LDA     #$06            ; entry mode set: cursor increment, no shift
        BSR     lcd_pwr_instr   ; write instruction (power-up)
        JSR     WAIT200us       ; <-----

        RTS
*==========================================================================
* Write instruction byte from accumulator during lcd powerup
lcd_pwr_instr:
	SEI			; disable interrupts
	MOV	#$FF,PORTC	; make sure IDE Interface is deselected

	MOV	#$FF,DDRA	; Command - Port A Outputs
	MOV	#$FF,DDRB	; Data    - Port B Outputs

	BCLR	LCDEN,PORTD	; EN = 0
	MOV	#$0F,PORTA	; RS, R/W = 0
	BCLR	LATCH,PORTD	
	BSET	LATCH,PORTD

	STA	PORTB		; instruction byte

        BSET    LCDEN,PORTD     ; E
        NOP                     ; Wait 1 us <-----
        BCLR    LCDEN,PORTD     ; E

	CLR	DDRA		; Port A Inputs (default)
	CLR	DDRB		; Port B Inputs (default)

	CLI			; re-enable interrupts

        JSR     WAIT100us       ; <-----
        RTS
*==========================================================================
* Write instruction byte from accumulator, use Wait for Busy
lcd_wr_instr:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Command - Port A Outputs
	MOV	#$FF,DDRB	; Data    - Port B Outputs

	BCLR	LCDEN,PORTD	; EN = 0
	MOV	#$0F,PORTA	; RS, R/W = 0
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD

	STA	PORTB		; instruction byte

        BSET    LCDEN,PORTD     ; E
        NOP                     ; Wait 1 us <-----
        BCLR    LCDEN,PORTD     ; E

	CLR	DDRA		; Port A Inputs (default)
	CLR	DDRB		; Port B Inputs (default)

	CLI			; re-enable interrupts

        BSR     WAIT_BUSY        ; <-----

        RTS
*==========================================================================
* Write Data byte from accumulator
lcd_wr_data:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Command - Port A Outputs
	MOV	#$FF,DDRB	; Data    - Port B Outputs

	BCLR	LCDEN,PORTD	; EN = 0
	MOV	#$4F,PORTA	; RS = 1, R/W = 0
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD

	STA	PORTB		; instruction byte

        BSET    LCDEN,PORTD     ; E
        NOP                     ; Wait 1 us <-----
        BCLR    LCDEN,PORTD     ; E

	CLR	DDRA		; Port A Inputs (default)
	CLR	DDRB		; Port B Inputs (default)

	CLI			; re-enable interrupts

        BSR     WAIT_BUSY        ; <-----

        RTS
*==========================================================================
* Check the Busy Flag
WAIT_BUSY:

* just do a timed wait (100us) without using X Reg
	LDA	#158	; 		3
wb100	DBNZA	wb100   ;		3
	RTS
*==========================================================================
* Machine cycle = 203.58 ns, 10 us = 49.152 bus cycles 
* xval = (10*4.9152 - 18)/3 = 10	(FBus = 4.9152 -> 203 ns)
* time = (10*3 +19)/4.9152 = 9.97 us
* 	JSR is 5 machine cycles		5
WAIT10us:
        STX    savex	;		3

	LDX	#158	; 		3
wt10	DBNZX	wt10    ;	3

	NOP		;		1
	LDX	savex   ;		3
	RTS		;		4
*==========================================================================
* Display a message from absolute location in H:X
* X = #MSG  (example:   LDHX    #MSG2 )
DOMSG:  LDA     ,X             ; LCD screen address
        JSR     lcd_wr_instr

dm3lp:  AIX	#$01
        LDA     ,X
        CMPA    #$FF
        BEQ     dm3out          ; we are done

        JSR     lcd_wr_data     ; put out the character
        BRA     dm3lp

dm3out: RTS
*===========================================================================
* Title and Copyright Messages
GREET_MSGS:
        LDHX    #MSG0           ; 'Design 99'
        JSR     DOMSG

        LDHX    #MSG1           ;' Project 99-10 '
        JSR     DOMSG

        JSR     WAITnClear

        LDHX    #MSG2           ; 'IDE CD-ROM '
        JSR     DOMSG

        LDHX    #MSG3           ; 'Controller '
        JSR     DOMSG

        JSR     WAITnClear

        RTS
*===========================================================================
MSG0:   FCB  $80,' IDE CD-ROM    ',$FF
MSG1:   FCB  $C0,' Controller    ',$FF
MSG2:   FCB  $80,'Copyright 1999 ',$FF
MSG3:   FCB  $C0,'Jed Margolin   ',$FF
MSG4:   FCB  $C1,'               ',$FF
MSG5:   FCB  $80,'   FLASH!      ',$FF
MSG6:   FCB  $80,'Error',$FF
MSG7:   FCB  $80,'Waiting For Ready ',$FF
MSG8:   FCB  $80,'Ready             ',$FF
MSG9:   FCB  $80,'Serial Port Mode  ',$FF
*==========================================================================
* Display Menu Messages
Display_Men_Msgs:
	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

        LDHX    #MSG0 		;
        JSR     DOMSG

        LDHX    #MSG1 		;
        JSR     DOMSG

	RTS
*===========================================================================
* Wait 1 sec, then clear the display
WAITnClear:
        JSR     WAIT1S

	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

	RTS
*---------------------------
* Wait 2 secs, then clear the display
WAITnCLR2:
        JSR     WAIT1S
	BRA	WAITnClear
*---------------------------
* Wait 3 secs, then clear the display
WAITnCLR3:
        JSR     WAIT1S
	BRA	WAITnCLR2
*===========================================================================
* Convert binary to bcd, 255. max
* uses ACC, Xreg, binl, bcd1, bcd0
* input from ACC, output in bcd1, bcd0
BIN_BCD_255:
        STA     binl
        CLR     bcd1
        CLR     bcd0
        LDX     #8
bcdloop:
        ROL     binl

	LDA	bcd0
	ADC	bcd0
	DAA
	STA	bcd0

	LDA	bcd1
	ADC	bcd1
	DAA
	STA	bcd1

        DECX
        BNE     bcdloop

        RTS
*===========================================================================
* Convert binary to bcd, 99. max
* uses ACC, Xreg, binl, bcd0
* input from ACC, output in bcd0
BIN_BCD_99:
        STA     binl
        CLR     bcd0
        LDX     #8
bcdlp:
	LSL	binl
	LDA	bcd0
	ADC	bcd0
	DAA
	STA	bcd0

abr1:   DECX
        BNE     bcdlp

        RTS
*============================================================================
* Convert binary to bcd: 99,999. max
* uses ACC, Xreg, binl, binh, bcd0,bcd1,bcd2
* input from binl,binh, output in bcd0,bcd1,bcd2
BIN_6BCD:
        CLR     bcd0
        CLR     bcd1
        CLR     bcd2

        LDX     #16
bcd6lp:
        ROL     binl
        ROL     binh

	LDA	bcd0
	ADC	bcd0
	DAA
	STA	bcd0

	LDA	bcd1
	ADC	bcd1
	DAA
	STA	bcd1

	LDA	bcd2
	ADC	bcd2
	DAA
	STA	bcd2

        DECX
        BNE     bcd6lp

        RTS
*==========================================================================
* Display BDC values
Display_6BCD:

	CLRH
	LDA	bcd2
	BEQ	dbcd1
        LDX     #$C0    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

dbcd1:	CLRH
	LDA	bcd1
	BEQ	dbcd2
        LDX     #$C2    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

dbcd2:	CLRH
	LDA	bcd0
        LDX     #$C4    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	RTS
*===========================================================================
* Display Binary from ACC, Xreg is cursor address
DISPLAY_BINARY:
        STA     temp
        TXA                     ; Set cursor
        JSR     lcd_wr_instr

        LDA     temp
        LSRA
        LSRA
        LSRA
        LSRA

        CMP     #$0A
        BPL     dbin1

        ADD     #$30
        BRA     dbin2

dbin1:  ADD     #$37
dbin2:  JSR     lcd_wr_data     ; first binary digit

        LDA     temp
        AND     #$0F

        CMP     #$0A
        BPL     dbin3

        ADD     #$30
        BRA     dbin4

dbin3:  ADD     #$37
dbin4:  JSR     lcd_wr_data     ; first binary digit
        RTS
*===========================================================================
* Display bcd from 0,X  ->bcdh, bcdl
* cursor from ACC
DISPLAY_BCD:
	CLRH				; we are using X as an address pointer

*       (LDA    #cursor)                ; Set cursor to first entry
        JSR     lcd_wr_instr

        LDA     1,X                     ; bcdh
        AND     #$0F
        ADD     #$30
        JSR     lcd_wr_data

        LDA    0,X                     ; bcdl
        LSRA
        LSRA
        LSRA
        LSRA
        ADD     #$30
        JSR     lcd_wr_data

        LDA     0,X                     ; bcdl
        AND     #$0F
        ADD     #$30
        JSR     lcd_wr_data

        RTS
*===========================================================================
* Display bcd from 0,X  ->bcdl
* cursor from ACC
DISPLAY_BCD2:
	CLRH				; we are using X as an address pointer

*       (LDA    #cursor)                ; Set cursor to first entry
        JSR     lcd_wr_instr

        LDA     0,X                     ; bcdl
        LSRA
        LSRA
        LSRA
        LSRA
        ADD     #$30
        JSR     lcd_wr_data

        LDA     0,X                     ; bcdl
        AND     #$0F
        ADD     #$30
        JSR     lcd_wr_data

        RTS
*===========================================================================
* Clear the Display
CLR_DISP:
        JSR     lcd_wr_instr
	LDX	#20
clrd1   LDA	#$20
	JSR     lcd_wr_data     ; put out the character
	DECX
	BNE	clrd1
	RTS
*===========================================================================
* Flash the LCD forever
FLASH_ERR:
* Clear the LCD
	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

* Wait 1 sec
flerr:  JSR     WAIT1S

* Put out a message
        LDHX    #MSG6		; 'Error'   
        JSR     DOMSG

* Wait 1 sec
        JSR     WAIT1S

* Clear the LCD
	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

	BRA	flerr
*===========================================================================
* Flash the LCD once
FLASH_LCD:
* Clear the LCD
	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

* Wait 1 sec
        JSR     WAIT1S

* Put out a message
        LDHX    #MSG5		; '   FLASH!     '   
        JSR     DOMSG

* Wait 1 sec
        JSR     WAIT1S

* Clear the LCD
	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

	RTS
*============================================================================
* Unsigned Divide: 16 bit dividend, 16 bit divisor
* Reuses Dividend for Quotient
* Input: dvdh, dvdl, dvsrh, dvsrl
* Requires three temporary locations: temph, templ, counter
UDIV:   MOV     #16,counter

        CLR     temph
        CLR     templ

	LSL     dvdl
        ROL     dvdh

udloop: ROL     templ
        ROL     temph
        LDA     templ
        SUB     dvsrl
        TAX
        LDA     temph
        SBC     dvsrh
        BLO     udv1

        STA     temph           ; positive result
        STX     templ

udv1:   ROL     dvdl            ; reuse dvdh, dvdl for quotient
        ROL     dvdh

        DEC     counter
        BNE     udloop
        NOP

        COM     dvdl
        COM     dvdh

        RTS
*===========================================================================
****************************************************************************
* Switch Functions
****************************************************************************
*===========================================================================
* Initialize the Switch data
INIT_SWITCHES:
	LDA	#$FF
	STA	sw1
	STA	sw2
	STA	sw3
	STA	sw4
	STA	sw5
	STA	sw6
	
	LDA	#$03
	STA	sw1+1
	STA	sw2+1
	STA	sw3+1
	STA	sw4+1
	STA	sw5+1
	STA	sw6+1

	RTS
*===========================================================================
* Read and debounce the switches during Timer Interrupts
DO_SWITCHES:

* Because we are using Port C and Port A, every other function that uses them
* *must* disable interrupts while using them.

* Read the switches
	MOV	#$FF,PORTC	; IDE Interface deselected
	CLR	DDRA		; Port A Inputs

	BCLR	SWEN,PORTC	; Switches selected
        LDA     PORTA
	BSET	SWEN,PORTC	; Switches deselected
	STA	swtemp		; switch: 0 = on

* sw1 =debounce register
* sw1+1: d2=positive transition, d1=old state, d0=debounced state, 
	LDHX	#sw1		
	BSR	swsub		; switch 1	

	LDHX	#sw2		
	BSR	swsub		; switch 2

	LDHX	#sw3		
	BSR	swsub		; switch 3

	LDHX	#sw4		
	BSR	swsub		; switch 4

	LDHX	#sw5		
	BSR	swsub		; switch 5

	LDHX	#sw6		
	BSR	swsub		; switch 6

	RTS
*----------------------------------
* debounce switch, check for positive transition 
swsub:
* get the two switch variables (it's faster this way)
	LDA	,X		
	STA	swtemp+1

	LDA	1,X
	STA	swtemp+2

* look at the switch
	ROR	swtemp		; switch state is in Carry
        ROL     swtemp+1	; debounce register for the particular switch
	BEQ	sws1		; all zero

	LDA	swtemp+1	; debounce register
	CMP	#$FF
	BNE	swsout		; not all zeros, not all ones: do nothing

	BSET	0,swtemp+2	; all ones: set swtemp+2(bit 0)
	BRA	sws2

sws1:	BCLR	0,swtemp+2	; all zeros: clear bit 0, swtemp+2

* switch is debounced, state is in swtemp+2(d0)
sws2:	BRSET	0,swtemp+2,sws3	; switch not pressed, update old switch state

* Switch is pressed, is it a positive transition?

* If the old switch was pressed, it is not a transistion
	BRCLR	1,swtemp+2,swsout	

* We have a winner, set the positive transition flag
* (Whoever uses the flag is responsible for clearing it)
	BSET	2,swtemp+2

* put new state into old state
sws3:   BRSET	0,swtemp+2,sws4
	BCLR	1,swtemp+2		
	BRA	swsout

sws4:	BSET	1,swtemp+2

* Write switch parameters back to the current switch
swsout:	LDA	swtemp+1		
	STA	,X

	LDA	swtemp+2
	STA	1,X

	RTS
*====================================================
* Look for switch transitions for changing the track number
SWTRANS:

* increment newtrack?
	BRCLR	2,sw4+1,csbr2		; switch: track +
	BCLR	2,sw4+1

	BSET	7,chtrack		; we have changed the track

	LDA	newtrack
	ADD	#$01
	CMP	track_count
	BLS	csbr1
	LDA	#$01	
csbr1:	STA	newtrack
	RTS

* decrement newtrack?
csbr2:	BRCLR	2,sw3+1,csbr4		; switch: track -
	BCLR	2,sw3+1

	BSET	7,chtrack	; we have changed the track

	LDA	newtrack
	SUB	#$01
	BNE	csbr3
	LDA	track_count
csbr3:	STA	newtrack

csbr4:  RTS
*====================================================
* Test Switches - On/Off
TEST_SW_A:
	LDA	#$80		; clear line 1
        JSR     CLR_DISP

	LDA	#$C0		; clear line 2
        JSR     CLR_DISP

	JSR	INIT_SWITCHES

tsw1lp:	JSR	DO_SWITCHES

	JSR	Display_Switch1		; display switch state

        JSR     WAIT10ms

	BRA	tsw1lp
*============================================================================
* Test Switches - Positive Transitions
TEST_SW_B:

	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

	JSR	INIT_SWITCHES

	CLR	pcount

tsw2lp:	JSR	DO_SWITCHES

	JSR	Display_Switch2		; positive switch transition

        JSR     WAIT10ms

	JSR	SWTRANS_tst		; do the switch transitions

	BRA	tsw2lp
*====================================================
* Display switch state (6 switches)
Display_Switch1		
	LDA	#$80
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	0,sw1+1,dswbr1
	LDA	#'0'
dswbr1: JSR     lcd_wr_data     ; put out the character

	LDA	#$83
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	0,sw2+1,dswbr2
	LDA	#'0'
dswbr2: JSR     lcd_wr_data     ; put out the character

	LDA	#$86
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	0,sw3+1,dswbr3
	LDA	#'0'
dswbr3: JSR     lcd_wr_data     ; put out the character

	LDA	#$89
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	0,sw4+1,dswbr4
	LDA	#'0'
dswbr4: JSR     lcd_wr_data     ; put out the character

	LDA	#$8C
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	0,sw5+1,dswbr5
	LDA	#'0'
dswbr5: JSR     lcd_wr_data     ; put out the character

	LDA	#$8F
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	0,sw6+1,dswbr6
	LDA	#'0'
dswbr6: JSR     lcd_wr_data     ; put out the character

	RTS
*===========================================================================
* Display positive switch transitions
Display_Switch2		
	LDA	#$80
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	2,sw1+1,dstbr1
	LDA	#'0'
dstbr1: JSR     lcd_wr_data     ; put out the character

	LDA	#$83
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	2,sw2+1,dstbr2
	LDA	#'0'
dstbr2: JSR     lcd_wr_data     ; put out the character

	LDA	#$86
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	2,sw3+1,dstbr3
	LDA	#'0'
dstbr3: JSR     lcd_wr_data     ; put out the character

	LDA	#$89
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	2,sw4+1,dstbr4
	LDA	#'0'
dstbr4: JSR     lcd_wr_data     ; put out the character

	LDA	#$8C
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	2,sw5+1,dstbr5
	LDA	#'0'
dstbr5: JSR     lcd_wr_data     ; put out the character

	LDA	#$8F
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	2,sw6+1,dstbr6
	LDA	#'0'
dstbr6: JSR     lcd_wr_data     ; put out the character

	RTS
*===================================================
* for testing Switch transitions
SWTRANS_tst:
	TST	pcount
	BEQ	swtbr1

	DEC	pcount
	BNE	swtbr3

* clear all the transition flags
	BCLR	2,sw1+1
	BCLR	2,sw2+1
	BCLR	2,sw3+1
	BCLR	2,sw4+1
	BCLR	2,sw5+1
	BCLR	2,sw6+1
	RTS

* look for a transition
swtbr1:	BRSET	2,sw1+1,swtbr2
	BRSET	2,sw2+1,swtbr2
	BRSET	2,sw3+1,swtbr2
	BRSET	2,sw4+1,swtbr2
	BRSET	2,sw5+1,swtbr2
	BRSET	2,sw6+1,swtbr2
	RTS

* we had a transition, start the counter
swtbr2:	MOV	#40,pcount

swtbr3:	RTS
*============================================================================
****************************************************************************
* Functions for the CDROM Drive
****************************************************************************
*==========================================================================
* CDROM Drive Hardware Reset
Hard_Reset:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Port A Output

	MOV	#$F7,PORTA
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; Reset Low

	JSR	WAIT100us

	MOV	#$FF,PORTA
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; Reset high

	CLR	DDRA		; Port A Input

	CLI			; re-enable interrupts

	RTS
*==========================================================================
*==========================================================================
* CDROM Drive Software Reset
Soft_Reset:

* Give Busy a chance to go low
	LDA	#$FF
	STA	pcount
sr1:	LDA	#$07			; read real status register
	STA	PortAddress
	JSR	ReadPort
	STA	status

	BRCLR	7,status,sr2		; Busy must be Low
	JSR	WAIT10ms
	DEC	pcount
	BNE	sr1

	LDA	#$FF			; error (status valid)
	RTS

* Write the command
sr2:	LDA	#$00			; device 0
	STA	datasave
	LDA	#DRIVE_SEL
	JSR	WritePort

	LDA	#SOFT_RESET_CMD
	STA	cmdsave
	JSR	Write_Command		; from cmdsave

* Give Busy a chance to go low
	LDA	#$FF
	STA	pcount
sr3:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRCLR	7,status,srout		; Busy must be Low
	JSR	WAIT10ms
	DEC	pcount
	BNE	sr3

	LDA	#$FF			; error (status valid)
	RTS

* status will be $00 until the first ATAPI command
* Do a Test_Unit_Ready (Get_Sense)
srout:	JSR	Get_Sense

	LDA	sense_error
	STA	status

	RTS
*============================================================================
*============================================================================
Show_Status:
	STX	binl

	LDA	#$07
	STA	PortAddress
	JSR	ReadPort
	STA	status
*	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC

	CLRH
        LDX     binl    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC
	RTS
*==========================================================================
*===========================================================================
* Get and display the Drive Information 
* Uses 'Drive Inquiry' and 'Read Drive Capabilities'
Drive_Info:
	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

	LDHX	#DIMSG0		; 'Drive Info'
	JSR	DOMSG

	JSR	WAIT1S
	JSR	WAIT1S

	JSR	Soft_Reset

* Inquiry
	CLRH
	JSR	Do_Inquiry
	BEQ	dibr1
	LDA	#$01			; error
	RTS

* Drive Capabilities
dibr1:	JSR	Get_Sense

	CLRH
	JSR	Drive_Caps_Cmd		; Get Drive Capabilities
	BEQ	dibr2
	LDA	#$02			; error
	RTS

* Show me the data
dibr2:	JSR	Show_RDC_All
	CLRA				; success
	RTS
*-----------------
DIMSG0:  FCB  $80,'Drive Info',$FF
DIMSG1:  FCB  $80,'Done',$FF
DIMSG2:  FCB  $80,'Error:',$FF
*===========================================================================
*==========================================================================
* Start_Disc Packet Command
Start_Disc:
	CLR	sense_error

* Form the command packet for Start Disc
	CLRA
	LDHX	#11			; X= 11->0 is packet[0..11]
sd1:	   STA	  buffer,X
	   DECX
	   BPL    sd1
      LDA	#START_STOP_EJECT
      STA	buffer+11

      MOV	#$01,buffer+7		; start the disc and read the TOC

      JSR	Do_Packet		; errlog in ACC
      
      RTS
*==========================================================================
*==========================================================================
Play_Track:
EndM	EQU	99	; End M
EndS	EQU	59	; End S
EndF	EQU	74	; End F

* Play Audio MSF from Track Number in ACC
Play_MSF:

* Get the MSF data from the TOC table
* Track number in ACC
	DECA			; track 1 = ptr 0
	LDX	#3		; 3 entries/track
	MUL			; low 8 bits in ACC
	TAX
	CLRH			; we are using X as an address pointer

* Get MSF
	LDA	cbuffer,X
	STA	Mval

	INCX
	LDA	cbuffer,X
	STA	Sval

	INCX
	LDA	cbuffer,X
	STA	Fval

* Form the command packet for PLAY_AUDIO_MSF
        MOV	#PLAY_AUDIO_MSF,buffer+11	; byte 0	Op Code
	CLR	buffer+10	; byte 1
	CLR	buffer+9	; byte 2
	MOV	Mval,buffer+8	; byte 3	Start M
	MOV	Sval,buffer+7	; byte 4	Start S
	MOV	Fval,buffer+6	; byte 5	Start F
	MOV	#EndM,buffer+5	; byte 6	Ending M
	MOV	#EndS,buffer+4	; byte 7	Ending S
	MOV	#EndF,buffer+3	; byte 8	Ending F
	CLR	buffer+2	; byte 9
	CLR	buffer+1	; byte 10
	CLR	buffer+0	; byte 11

	JSR	Do_Packet	; returns $00 = no errors
	BEQ	pmout
	JSR	FLASH_LCD	; flash it once

* done
pmout:	RTS
*==========================================================================
*==========================================================================
TOC10:	FCB   $00,$00		; 11,10
	FCB   $80,$00		; 9,8
	FCB   $20,$01		; 7,6
	FCB   $00,$00		; 5,4
	FCB   $00,$10		; 3,2
	FCB   $02,READ_TOC	; 1,0

* Read Table of Contents 10 (for audio CD tracks)
Make_TOC10:

	CLR	track_count
	CLR	track_ptr

* Form the command packet for Read TOC 10
	LDHX	#11			; X= 11->0 is packet[0..11]
toc1:	   LDA	  TOC10,X
	   STA	  buffer,X
	   DECX
	   BPL    toc1

      JSR	Do_Packet		; returns $00 = no errors
      BEQ	toc2
      JMP	FLASH_ERR

* wait while BSY
toc2:

* get and decode the data
	JSR     READ_CD_WORD	; length

	MOV	byte_low,length+1	; length MSB
	MOV	byte_high,length+0	; length LSB

	LDA	length+0	; subtract 4+11
	ADD	#$F1
	STA	length+0

	LDA	length+1
	ADC	#$FF
	STA	length+1

	JSR     READ_CD_WORD	; first, last session (ignore)

* There are 11 bytes/record
toclp:	JSR     READ_CD_WORD	; Session #, ADR:Control (ignore)

	JSR     READ_CD_WORD	; TNO (ignore), Point

	LDA	byte_high	; 0-99 is track number
	STA	track
	CMP	#99
	BMI	toc3	

* not a track number 
	JSR     READ_CD_WORD	; byte 4, 5
	JSR     READ_CD_WORD	; byte 6, 7
	JSR     READ_CD_WORD	; byte 8, 9 
	JSR     READ_CD_WORD    ; byte 10, 0 (byte_high is byte 0 next record)
	BRA	toc_odd

* we have a track number
toc3:
	JSR     READ_CD_WORD	; byte 3, byte 4 (ignore)
	JSR     READ_CD_WORD	; byte 5, byte 6 (ignore)

	JSR     READ_CD_WORD	; M, S
	MOV	byte_low,Mval
	MOV	byte_high,Sval

	JSR     READ_CD_WORD	; F, first bute of next record
	MOV	byte_low,Fval

	JSR	Log_MSF

* byte_high is first byte of next record
toc_odd:
	JSR	Do_Length
	BPL	toc4		; msb less than zero means we are done

* we are done, so just read and discard 2048 bytes
	JSR	FLUSH
	RTS

* byte_high is byte 0 of next record, ignore it
toc4:	JSR     READ_CD_WORD    ; byte 1, 2 (ignore)

	JSR     READ_CD_WORD	; byte 3 Point, byte 4 (ignore)

	LDA	byte_low	; byte 3: 0-99 is track number
	STA	track
	CMP	#99
	BMI	toc5	

* not a track number 
	JSR     READ_CD_WORD	; byte 5, byte 6
	JSR     READ_CD_WORD	; byte 7, byte 8
	JSR     READ_CD_WORD	; byte 9, byte 10
	JMP	toclp		; we are at an even boundary again

toc5:	JSR     READ_CD_WORD	; byte 5, byte 6 (ignore)

	JSR     READ_CD_WORD	; byte 7, byte 8 (Mvalue)
	MOV	byte_high,Mval

	JSR     READ_CD_WORD	; S, F
	MOV	byte_low,Sval
	MOV	byte_high,Fval

	JSR	Log_MSF

	JSR	Do_Length
	BMI	toc6		; msb less than zero means we are done
	JMP	toclp

*  we will just read and discard 2048 bytes
toc6:	JSR	FLUSH
	RTS
*==========================================================================
* Add the MSF data to the buffer (for making TOC table)
Log_MSF:
	CLRH			; we are using X as an address pointer
	LDX	track_ptr

	LDA	Mval
	STA	cbuffer,X
	INCX

	LDA	Sval
	STA	cbuffer,X
	INCX

	LDA	Fval
	STA	cbuffer,X
	INCX

	STX	track_ptr

	INC	track_count

	RTS
*==========================================================================
*===========================================================================
* Get Drive Capabilities, return data in cbuffer
Drive_Caps_Cmd:

* Make the packet
	JSR	Get_Caps

* Send Packet Command
	JSR	Packet_Command		; send Packet Command
	BEQ	drcbr1

	LDA	#$01			; Busy never went low
	RTS

* Give DRQ a chance to go high
drcbr1:	MOV	#$40,pcount
drclp2:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRSET	3,status,drcbr2		; DRQ must be high
	JSR	WAIT1ms
	DEC	pcount
	BNE	drclp2

	LDA	#$02			; DRQ never went high
	RTS

* Send the Packet
drcbr2:	JSR	Write_DBuffer   	; returns status in ACC

* Give Busy a chance to go low
	MOV	#$40,pcount
drcct1:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRCLR	7,status,drcct2		; Busy must be Low
	JSR	WAIT1ms
	DEC	pcount
	BNE	drcct1

	LDA	#$03			; Busy never went low
	RTS

drcct2:	

* Give DRQ a chance to go high
	MOV	#$40,pcount
drcct3:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRSET	3,status,drcct4		; DRQ must be high
	JSR	WAIT1ms
	DEC	pcount
	BNE	drcct3

	LDA	#$04		; DRQ did not go high
	RTS

* Get the data (20 bytes: Goes into cbuffer)
drcct4:	JSR	GCDATA
	BEQ	drcct5

	LDA	#$06		; bad news, no data
	RTS

drcct5: CLRA			; success
	RTS
*==========================================================================
*==========================================================================
RDC:	FCB   $00,$00		  ; 11,10
	FCB   $00,$20		  ; 9,8   0,Length LSB
	FCB   $00,$00		  ; 7,6   
	FCB   $00,$00		  ; 5,4   
	FCB   $00,$AA		  ; 3,2   0, PC/Page Code
	FCB   $00,MODE_SENSE      ; 1,0

* Make Command Buffer for 'Read Drive Capabilities'
Get_Caps:
	LDHX	#11			; X= 11->0 is packet[0..11]
rdc1:	   LDA	  RDC,X
	   STA	  buffer,X
	   DECX
	   BPL    rdc1
       RTS
*==========================================================================
* Get the data from 'Read Drive Capabilities'
GCDATA:	LDX	#100		; timeout if we don't find the real data
	
gcd1:	JSR	READ_CD_WORD

	LDA	#$2A		; Real data starts with $2A

	CMP	byte_low
	BEQ	gcd2

	CMP	byte_high
	BEQ	gcd4

	DECX
	BNE	gcd1

	LDA	#10		; didn't get it
	RTS

* started with low byte
gcd2:	CLRX
	CLRH

	LDA	byte_low
	STA	cbuffer,X
	INCX

	LDA	byte_high
	STA	cbuffer,X
	INCX

* do 9 more words (18 bytes)
	MOV	#9,vcount

gcd3:	JSR	READ_CD_WORD

	LDA	byte_low
	STA	cbuffer,X
	INCX

	LDA	byte_high
	STA	cbuffer,X
	INCX

	DEC	vcount
	BNE	gcd3

	RTS			; returns with ACC=0 (success)

* started with high byte
gcd4:	CLRX
	CLRH

	LDA	byte_high
	STA	cbuffer,X
	INCX

* do 9 more words (18 bytes)
	MOV	#9,vcount

gcd5:	JSR	READ_CD_WORD

	LDA	byte_low
	STA	cbuffer,X
	INCX

	LDA	byte_high
	STA	cbuffer,X
	INCX

	DEC	vcount
	BNE	gcd5

	JSR	READ_CD_WORD	; one more byte

	LDA	byte_low
	STA	cbuffer,X

	CLRA
	RTS			; returns with ACC=0 (success)
*==========================================================================
* Messages for Drive Info
DMSG0:  FCB  $C0,'CD-R Read:',$FF
DMSG1:  FCB  $C0,'CD-RW Read:',$FF

DMSG2:  FCB  $C0,'CD-R Write:',$FF	
DMSG3:  FCB  $C0,'CD-RW Write',$FF	

DMSG4:  FCB  $C0,'Mode 2 Form 1:',$FF
DMSG5:  FCB  $C0,'Mode 2 Form 2:',$FF
DMSG6:  FCB  $C0,'Multi Session:',$FF	
DMSG7:  FCB  $C0,'Audio Play:',$FF	

DMSG8:  FCB  $CF,'Yes',$FF
DMSG9:  FCB  $CF,'No',$FF

DMSG10:  FCB  $83,'Max Speed:',$FF	
DMSG11:  FCB  $C8,'KB/sec',$FF		
DMSG12:  FCB  $83,'Buffer Size:',$FF	
DMSG13:  FCB  $CA,'KBytes',$FF		

DMSG14:  FCB  $CE,'Done',$FF
*--------------------------------
* Show some Read_Drive_Capabilities
Show_RDC:

	CLR	pcount

* 'CD-R Read'
	JSR	Show_Info	; pcount comes back incremented
	JSR	WAITnCLR2	; wait 2 seconds, then clear the display

* 'CD-RW Read'
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'CD-R Write'	
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'CD-RW Write'	
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'Mode 2 Form 1'
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'Mode 2 Form 2'
	JSR	Show_Info		
	JSR	WAITnCLR2
	
* 'Multi Session'	
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'Audio Play'	
	JSR	Show_Info		
	JSR	WAITnCLR2

	RTS			; we are done	
*--------------------------------
* Show Read_Drive_Capabilities
Show_RDC_All:

	CLR	pcount

* 'CD-R Read'
	JSR	Show_Info	; pcount comes back incremented
	JSR	WAITnCLR2	; wait 2 seconds, then clear the display

* 'CD-RW Read'
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'CD-R Write'	
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'CD-RW Write'	
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'Mode 2 Form 1'
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'Mode 2 Form 2'
	JSR	Show_Info		
	JSR	WAITnCLR2
	
* 'Multi Session'	
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'Audio Play'	
	JSR	Show_Info		
	JSR	WAITnCLR2

* 'Max Speed'	
	JSR	Show_Speed	; Show the Speed data
	JSR	WAITnCLR2	

* 'Buffer Size'
	JSR	Show_Bsize	; Show the Buffer Size data
	JSR	WAITnCLR2	

	RTS			; we are done	
*==========================================================================
*		Message (Word), Data Address (Word), Mask (Byte), 

Info_Table:   
      FDB	DMSG0,cbuffer+2  
      FCB	$01		; CD-R Read

      FDB	DMSG1,cbuffer+2  
      FCB	$02		; CD-RW Read

      FDB	DMSG2,cbuffer+3  
      FCB	$01		; CD-R Write

      FDB	DMSG3,cbuffer+3  
      FCB	$02		; CD-RW Write

      FDB	DMSG4,cbuffer+4  
      FCB	$10		; CD-R Write

      FDB	DMSG5,cbuffer+4  
      FCB	$20		; CD-RW Write

      FDB	DMSG6,cbuffer+4  
      FCB	$10		; CD-R Write

      FDB	DMSG7,cbuffer+4  
      FCB	$20		; CD-RW Write
*------------------------------------
* Do the Drive Info_Table
* pcount is index to Info_Table, comes back incremented
Show_Info:

* get the offset of the table entry
	LDX	pcount
	LDA	#5		; 5 bytes/entry
	MUL
	TAX

	CLRH

* address of message
	LDA	Info_Table+0,X
	STA	address1+0	; msb

	LDA	Info_Table+1,X
	STA	address1+1	; lsb

* address of buffer data
	LDA	Info_Table+2,X
	STA	address2+0	; msb

	LDA	Info_Table+3,X
	STA	address2+1	; lsb

* mask
	LDA	Info_Table+4,X
	STA	mask

* Do the message
	LDHX	address1	; address of table entry
	JSR	DOMSG

* Get the data and use the data mask
	LDHX	address2	; address of buffer data
	LDA	,X   		; buffer data
	AND	mask
	JSR	YesNo

	INC	pcount
	RTS
*----------------------------
* Display 'yes' or 'no' depending in Z Flag
* Zero Flag: 1 = yes, 0 = no
YesNo:	BEQ	no

yes:	LDHX	#DMSG8		; yes
	BRA	ynbr

no:	LDHX	#DMSG9		; no

ynbr:   JSR     DOMSG

	RTS
*----------------------------
* Show the Speed data
Show_Speed
	LDHX	#DMSG10		; Speed
	JSR     DOMSG

	LDA	cbuffer+8	; msb
	STA	binh
	LDA	cbuffer+9	; lsb
	STA	binl

	JSR	Bin_6BCD	; convert 16 bit binary to 5 BCD digits
	JSR	Display_6BCD

	LDHX	#DMSG11		; KB/secs
	JSR     DOMSG

	RTS
*----------------------------
* Show the Buffer Size data
Show_BSize
	LDHX	#DMSG12		; Speed
	JSR     DOMSG

	LDA	cbuffer+12	; msb
	STA	binh
	LDA	cbuffer+13	; lsb
	STA	binl

	JSR	Bin_6BCD	; convert 16 bit binary to BCD
	JSR	Display_6BCD

	LDHX	#DMSG13		; KB/secs
	JSR     DOMSG

	RTS
*==========================================================================
*==========================================================================
INQ:	FCB   $00,$00		; 11,10
	FCB   $00,$00		; 9,8
	FCB   $00,$00		; 7,6
	FCB   $00,96		; 5,4    4=allocation
	FCB   $00,$00		; 3,2
	FCB   $00,INQUIRY	; 1,0

* Do the Inquiry command and report results
Do_Inquiry:

	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

* Form the command packet for Read TOC 10
	CLRH
	LDHX	#11			; X= 11->0 is packet[0..11]
inq1:	   LDA	  INQ,X
	   STA	  buffer,X
	   DECX
	   BPL    inq1

      JSR	Do_Packet		; returns $00 = no errors
      BEQ	inq2
      JMP	FLASH_ERR

* wait while BSY
inq2:

* get and decode the data
	JSR     READ_CD_WORD	; pass
	JSR     READ_CD_WORD	; pass
	JSR     READ_CD_WORD	; pass
	JSR     READ_CD_WORD	; pass

* Vendor
	LDHX	#QMSG0		; Vendor
	JSR     DOMSG

	LDA	#$C0		; start of second line
        JSR     lcd_wr_instr

	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word

	JSR	WAITnCLR3

* Product
	LDHX	#QMSG1		; Model
	JSR     DOMSG

	LDA	#$C0	; start of second line
        JSR     lcd_wr_instr

	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word

	JSR	WAITnCLR3

* Product Rev Level
	LDHX	#QMSG2		; Rev
	JSR     DOMSG

	LDA	#$85	
        JSR     lcd_wr_instr

	JSR	RnD		; Read and display a word
	JSR	RnD		; Read and display a word

	JSR	WAITnCLR3

	JSR	FLUSH

	RTS
*==========================================================================
* Read and display a word
RnD:	JSR     READ_CD_WORD

	LDA	byte_low
	JSR	lcd_wr_data

	LDA	byte_high
	JSR	lcd_wr_data

	RTS
*------------------------------------
QMSG0:  FCB  $80,'Vendor:',$FF
QMSG1:  FCB  $80,'Model:',$FF
QMSG2:  FCB  $80,'Rev:',$FF
*==========================================================================
* Time comes from Reading the SubCode
Show_Time:
	LDA	buffer+6		; Track
	JSR	BIN_BCD_99
	LDA	bcd0
	LDX	#$86
	CLRH
	JSR	DISPLAY_BINARY

	LDA	buffer+13		; Relative M
	JSR	BIN_BCD_99
	LDA	bcd0
	LDX	#$8B
	CLRH
	JSR	DISPLAY_BINARY

	LDA	buffer+14		; Relative S
	JSR	BIN_BCD_99
	LDA	bcd0
	LDX	#$8E
	CLRH
	JSR	DISPLAY_BINARY

	LDA	buffer+9		; Absolute M
	JSR	BIN_BCD_99
	LDA	bcd0
	LDX	#$CB
	CLRH
	JSR	DISPLAY_BINARY

	LDA	buffer+10		; Absolute S
	JSR	BIN_BCD_99
	LDA	bcd0
	LDX	#$CE
	CLRH
	JSR	DISPLAY_BINARY

	RTS
*==========================================================================
Time_Header:
	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

	LDHX	#TMSG0		; 'Track:'
        JSR     DOMSG

	LDHX	#TMSG1		; 'Abs Time:'
        JSR     DOMSG

	LDA	#$8D
        JSR     lcd_wr_instr
	LDA	#':'
        JSR     lcd_wr_data

	LDA	#$CD
        JSR     lcd_wr_instr
	LDA	#':'
        JSR     lcd_wr_data

	RTS

TMSG0:  FCB  $80,'Track:',$FF
TMSG1:  FCB  $C0,'Abs Time:',$FF
*==========================================================================
* Read SubCode to get the current time
Read_SubCode:
	JSR	Make_Sub_Packet		; Make SubCode Command Packet

* Send Packet Command
	JSR	Do_Packet
	BEQ	rscbr1

	LDA	#$01		; Busy never went low
	RTS

* Get the data
rscbr1:	JSR	Read_Sub_Data
	RTS
*==========================================================================
*==========================================================================
SUB:	FCB   $00,$00		  ; 11,10
	FCB   $00,$10		  ; 9,8   Length MSB,LSB
	FCB   $00,$00		  ; 7,6   
	FCB   $00,$00		  ; 5,4   
	FCB   $01,$40		  ; 3,2   Format, $40
	FCB   $02,SUBCHAN         ; 1,0   MSF,Cmd

* Make Command Packet for 'Read Sub Channel'
Make_Sub_Packet:
	LDHX	#11			; X= 11->0 is packet[0..11]
gsp1:	   LDA	  SUB,X
	   STA	  buffer,X
	   DECX
	   BPL    gsp1
       RTS
*==========================================================================
* Get the data from 'Read SubChannel'
Read_Sub_Data:

	CLRX
	CLRH

* do 8 words (16 bytes)
	MOV	#8,vcount

rsb1:	JSR	READ_CD_WORD

	LDA	byte_low
	STA	buffer,X
	INCX

	LDA	byte_high
	STA	buffer,X
	INCX

	DEC	vcount
	BNE	rsb1

	RTS
*==========================================================================
* Do the Stop command
Do_STOP:

* Form the command packet for Read TOC 10
	LDHX	#11			; X= 11->0 is packet[0..11]
stp1:	   LDA	  #00
	   STA	  buffer,X
	   DECX
	   BPL    stp1

      LDA	#STOP
      STA	buffer+11

      JSR	Do_Packet		; returns $00 = no errors
      RTS
*==========================================================================
* send Packet Command
* If ACC returns with $FF, BUSY never went low
* Otherwise, ACC returns $00, with status register in status
Packet_Command:

* Give Busy a chance to go low
	MOV	#$20,pcount
ct3a:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRCLR	7,status,ct3b		; Busy must be Low
	JSR	WAIT1ms
	DEC	pcount
	BNE	ct3a

	LDA	#$FF			; Busy never went low
	RTS

* Write the command
ct3b:	MOV	#ATAPI_PACKET_CMD,cmdsave
	JSR	Write_Command		; from cmdsave

* read the real status register (resets INTRQ)
	MOV	#$07,PortAddress
	JSR	ReadPort
	STA	status
	CLRA
	RTS
*==========================================================================
* Send 'Packet Command' and Packet Data from buffer
* Returns in ACC (also errlog):
*   $00    No Errors
*   $FE    Before 'Send Packet Command' BUSY never went low, command not sent
*   $FD    After 'Send Packet Command' DRQ never went high
*   $FC    After sending Packet data BUSY never went low
*
Do_Packet:
	CLR	errlog

* Send Packet Command
*    If ACC returns with $01, BUSY never went low
*    Otherwise, ACC returns $00, with status register in status
	JSR	Packet_Command	     ; send Packet Command
	BEQ	dp2
	LDA	#$01
	STA	errlog
	RTS		        ; $01 = BUSY never went low, command not sent

* Packet command was sent

* Give DRQ a chance to go high
dp2:	LDA	#$20
	STA	pcount
dp3:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRSET	3,status,dp4		; DRQ must be high
	JSR	WAIT1ms
	DEC	pcount
	BNE	dp3

	LDA	#$02			; DRQ never went high
	RTS

* Send the Packet
dp4:	JSR	Write_DBuffer   	; returns status in ACC

* Packet command and packet data were sent

* Give Busy a chance to go low
	MOV	#$40,pcount
dp5:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRCLR	7,status,dp6		; Busy is Low
	JSR	WAIT100ms
	DEC	pcount
	BNE	dp5

	LDA	#$03			; Busy never went low
	STA	errlog
	RTS

dp6:	CLRA
	RTS
*==========================================================================
* Fast Forward
* Read ACC number of words, only the last one will be reported
FFORWARD:
      STA	wcount
ffbr: JSR   	READ_CD_WORD   	; read two bytes
      DEC	wcount
      BNE	ffbr
      RTS	  
*==========================================================================
* Flush any remaining data in the block (flush 2048 bytes to make sure)
FLUSH:	LDA	#$00
	JSR	FFORWARD
	LDA	#$00
	JSR	FFORWARD
	LDA	#$00
	JSR	FFORWARD
	LDA	#$00
	JSR	FFORWARD

	RTS
*==========================================================================
****************************************************************************
* Lower Level CDROM Drive Functions
****************************************************************************
*    d7     d6     d5    d4   d3   d2  d1  d0
*   LCDRW  LCDRS             /Res  A2  A1  A0
* CDROM subroutines
* Returns Alt_Status in status as well as ACC
* uses ACC, Xreg
* Alt_Status Register is $6
Get_Alt_Status:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Port A Output

	MOV	#$FE,PORTA	; register address $6
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; address is set
	
	BCLR	CS1,PORTC	; /CS1 low

	BCLR	RD,PORTC	; /RD low

	NOP
	CLR	DDRA		; Port A Input

	LDA	PORTA
	STA	status

	BSET	RD,PORTC	; /RD high

	BSET	CS1,PORTC	; /CS1 high

	CLI			; re-enable interrupts

	RTS
*============================================================================
*    d7     d6     d5    d4   d3   d2  d1  d0
*   LCDRW  LCDRS             /Res  A2  A1  A0
* PORTA is to be Input by default
* Read a selected CDROM Port
* port number in ACC
ReadPort:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Port A Output

	LDA	PortAddress
	AND	#$07		; address is low 3 bits
	ORA	#$F8		; keep everthing else high
	STA	PORTA		
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; address is set

	BCLR	CS0,PORTC	; /CS0 low

	BCLR	RD,PORTC	; /RD low

	NOP
	CLR	DDRA		; Port A Input

	LDA	PORTA
	STA	status

	BSET	RD,PORTC	; /RD high

	BSET	CS0,PORTC	; /CS0 high

	CLI			; re-enable interrupts

	RTS
*============================================================================
*    d7     d6     d5    d4   d3   d2  d1  d0
*   LCDRW  LCDRS             /Res  A2  A1  A0
* Read CDROM Data port (word) and send it back to the PC
DoData1:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Port A Output

	MOV	#$F8,PORTA	; address 0
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; address is set

	CLR	DDRA		; Port A Input
	CLR	DDRB		; Port B Input

	BCLR	CS0,PORTC	; /CS0 low

	BCLR	RD,PORTC	; /RD low

	NOP

	JSR	WAIT1ms
	LDA	PORTA
	JSR	SENDCHAR

	JSR	WAIT1ms
	LDA	PORTB
	JSR	SENDCHAR

	BSET	RD,PORTC	; /RD high

	BSET	CS0,PORTC	; /CS0 high

	CLI			; re-enable interrupts

	RTS
*============================================================================
*    d7     d6     d5    d4   d3   d2  d1  d0
*   LCDRW  LCDRS             /Res  A2  A1  A0
* Write ACC to CDROM Command Port
Write_Command:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Port A Output

	MOV	#$FF,PORTA	; address 7
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; address is set

	BCLR	CS0,PORTC	; /CS0 low

	BCLR	WR,PORTC	; /WR low

	MOV	cmdsave,PORTA

	BSET	WR,PORTC	; /WR high
	BSET	CS0,PORTC	; /CS0 high

	CLR	DDRA		; Port A back to inputs

	CLI			; re-enable interrupts

	RTS
*============================================================================
*    d7     d6     d5    d4   d3   d2  d1  d0
*   LCDRW  LCDRS             /Res  A2  A1  A0
* Write packet from buffer into CDROM data port
Write_DBuffer:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Port A Output
	MOV	#$FF,DDRB	; Port B Output

	MOV	#$F8,PORTA	; address 0
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; address is set

	LDHX	#11
wdblp:	   LDA	buffer,x
	   STA	PORTA
	   DECX

	   LDA	buffer,x
	   STA	PORTB

	   BCLR	   CS0,PORTC	; /CS0 low
	   BCLR	   WR,PORTC	; /WR low
	   BSET	   WR,PORTC	; /WR high
	   BSET	   CS0,PORTC	; /CS0 high

	   DECX
	   BPL	wdblp


	CLR	DDRA		; Port A back to inputs
	CLR	DDRB		; Port B back to inputs

	CLI			; re-enable interrupts

	RTS
*============================================================================
*    d7     d6     d5    d4   d3   d2  d1  d0
*   LCDRW  LCDRS             /Res  A2  A1  A0
READ_CD_WORD:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Port A Output

	MOV	#$F8,PORTA	; address 0
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; address is set

	CLR	DDRA		; Port A inputs
	CLR	DDRB		; Port B inputs

	BCLR	CS0,PORTC	; /CS0 low
	BCLR	RD,PORTC	; /RD low

	LDA	PORTA
	STA	byte_low

	LDA	PORTB
	STA	byte_high

	BSET	RD,PORTC	; /RD high
	BSET	CS0,PORTC	; /CS0 high

	CLI			; re-enable interrupts

	RTS
*============================================================================
TestRD_CD:
	LDA	temp
	STA	byte_low
	INCA
	STA	byte_high	
	INCA
	STA	temp
	RTS
*============================================================================
* Returns Error Code in status as well as ACC
* uses ACC, Xreg
* Error Reg is b001
*    d7    d6   d5    d4    d3  d2  d1  d0 
*   /Res  /RD  /WR  /CS1  /CS0  A2  A1  A0
Get_Error_Rd:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Port A Output

	MOV	#$F9,PORTA	; register $1		

	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; address is set

	CLR	DDRA		; Port A Input

	BCLR	CS0,PORTC	; /CS0 low

	BCLR	RD,PORTC	; /RD low

	NOP

	LDA	PORTA
	STA	status

	BSET	RD,PORTC	; /RD high

	BSET	CS0,PORTC	; /CS0 high

	CLI			; re-enable interrupts

	RTS
*============================================================================
*    d7    d6   d5    d4    d3  d2  d1  d0
*   /Res  /RD  /WR  /CS1  /CS0  A2  A1  A0
* Address from ACC, data from datasave
WritePort:
	SEI			; disable interrupts

	MOV	#$FF,DDRA	; Port A Output

	LDA	PortAddress
	AND	#$07		; address is low 3 bits
	ORA	#$F8		; keep everthing else high
	STA	PORTA		
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; address is set

	LDA	datasave
	STA	PORTA		; data

	BCLR	CS0,PORTC	; /CS0 low
	BCLR	WR,PORTC	; /WR low
	BSET	WR,PORTC	; /WR high
	BSET	CS0,PORTC	; /CS0 high

	CLR	DDRA		; Port A back to inputs

	CLI			; re-enable interrupts

	RTS
*==========================================================================
* Wait for CDROM ready
* Returns: ACC returns 0=ready, $FF=timed out
Wait_For_Ready:

	MOV	#30,wfrcount		; 30 seconds 

wfr1:		JSR	Get_Sense	; get the sense keys
		BEQ	wfr2		; sense code = 0

		LDA	#$80		; clear the first line
		JSR     CLR_DISP
		LDHX    #MSG7           ; 'Waiting For Ready'
		JSR     DOMSG

		JSR	WAIT250ms
		JSR	WAIT250ms

		JSR	Get_Sense	; get the sense keys
		BEQ	wfr2		; sense code = 0

		LDA	#$80		; clear the first line
		JSR     CLR_DISP
		JSR	WAIT250ms
		JSR	WAIT250ms

		DEC	wfrcount
		BNE	wfr1

	MOV	#$FF,wready_error		; we timed out

	LDA	#$80		; clear the first line
	JSR     CLR_DISP

	RTS

wfr2:	LDA	#$80		; clear the first line
	JSR     CLR_DISP

	CLRA				; unit ready
	STA	wready_error
	RTS
*==========================================================================
* Get the sense keys
Get_Sense:
	CLR	sense_error

* Form the command packet for TEST_UR
	CLRA
	LDHX	#11			; X= 11->0 is packet[0..11]
gs1:	   STA	  buffer,X
	   DECX
	   BPL    gs1
      LDA	#TEST_UR
      STA	buffer+11

* Send Packet Command
*    If ACC returns with $FE, BUSY never went low
*    Otherwise, ACC returns $00, with status register in status
	JSR	Packet_Command	     ; send Packet Command
	BEQ	gs2
	LDA	#$FE
	STA	sense_error
	RTS		        ; $FE = BUSY never went low, command not sent

* Give DRQ a chance to go high
gs2:	MOV	#$20,pcount
gs3:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRSET	3,status,gs4		; DRQ must be high
	JSR	WAIT1ms
	DEC	pcount
	BNE	gs3
	LDA	#$FD		; $FD = DRQ never went high
	STA	sense_error
	RTS

* Send the Packet
gs4:	JSR	Write_DBuffer

* Packet command and packet data were sent

* Give Busy a chance to go low
	MOV	#$20,pcount
gs5:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRCLR	7,status,gs6		; Busy is Low
	JSR	WAIT1ms
	DEC	pcount
	BNE	gs5

	LDA	#$FC			; Busy never went low
	STA	sense_error
	RTS

* Read the Error Register
gs6:	JSR	Get_Error_Rd	; Returns Error code in status as well as ACC
	LSRA
	LSRA
	LSRA
	LSRA
	STA	sense
	RTS			; returns sense bits in ACC: 3 2 1 0
*==========================================================================
* Wait for DRQ
Wait_For_DRQ:
	LDA	#$40
	STA	pcount
wdbr1:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRSET	3,status,wdbr2		; DRQ must be high
	JSR	WAIT1ms
	DEC	pcount
	BNE	wdbr1

	LDA	#$02			; DRQ never went high
	RTS
wdbr2:  CLRA
	RTS
*==========================================================================
******************************************************************************
* Functions for ISO-9660 CD Data Discs
******************************************************************************
*==========================================================================
* Read_LBA command Packet 
CMD_READ_LBA:
	CLR	sense_error

	MOV	#READ_DATA10,buffer+11	; op code		byte 0
	CLR	buffer+10		; Reserved		byte 1
	MOV	LBA_Adr+3,buffer+9	; LBA Byte 3		byte 2
	MOV	LBA_Adr+2,buffer+8	; LBA Byte 2		byte 3
	MOV	LBA_Adr+1,buffer+7	; LBA Byte 1		byte 4
	MOV	LBA_Adr+0,buffer+6	; LBA Byte 0		byte 5
	CLR	buffer+5		; Reserved		byte 6
	CLR	buffer+4		; Transfer Length MSB	byte 7
	MOV	#$01,buffer+3		; Transfer Length LSB	byte 8
	CLR	buffer+2		; Reserved		byte 9
	CLR	buffer+1		; Reserved		byte 10
	CLR	buffer+0		; Reserved		byte 11

        JSR	Do_Packet		; errlog in ACC

	RTS
*==========================================================================
* Get and report 2048 bytes of data
Report_2kData:
	CLR	LBA_Count
	MOV	#$04,LBA_Count+1	; $400 words = $800 (2048) bytes

rkdbr:	   JSR	   GETCHAR		; wait for PC

	   JSR	   READ_CD_WORD

	   LDA	   byte_low
	   JSR	   SENDCHAR

	   JSR	   GETCHAR		; wait for PC
	   LDA	   byte_high
	   JSR	   SENDCHAR

	   LDA	   LBA_Count
	   ADD	   #$FF
	   STA	   LBA_Count

	   LDA	   LBA_Count+1
	   ADC	   #$FF
	   STA	   LBA_Count+1

	   BNE	   rkdbr		; high byte not zero

	   TST	   LBA_Count
	   BNE	   rkdbr		; high byte not zero

* We are done
      RTS
*==========================================================================
* Initialize data buffer pointers
Init_BPtrs:
	CLR	LBA_Count
	CLR	LBA_Count+1
	CLR	CBUFF_Count
	CLR	CBUFF_In_Ptr
	CLR	CBUFF_Out_Ptr
	RTS
*---------------------------------------------------------
* Get LBA for the Root Directory 
* Root Directory LBA is at bytes 158-162
* Primary Descriptor Volume must already have been issued
Get_Dir_LBA:

* Sift through the data

* discard 158 bytes
	MOV	#79,vcount

gdbr1:  JSR     READ_CD_WORD		; discard it
	DEC	vcount
	BNE	gdbr1

	JSR     READ_CD_WORD		; Directory LBA

	MOV	byte_low,LBA_Adr+0
	MOV	byte_high,LBA_Adr+1

	JSR     READ_CD_WORD		; Directory LBA

	MOV	byte_low,LBA_Adr+2
	MOV	byte_high,LBA_Adr+3

	JSR	Flush
	RTS
*==========================================================================
* Send the cbuffer to the PC
Send_Buffer:
	CLR	bcount
	CLRH			; we are using X as an address pointer

	LDA	#$AA		; tell PC we are ready
	JSR	SENDCHAR

sbf1:	JSR	GETCHAR
	TSTA
	BEQ	sbfend

	LDX	bcount
	CLRH
	LDA	cbuffer,X
	JSR	SENDCHAR

	INC	bcount
	BRA	sbf1

sbfend: RTS
*==========================================================================
* Send buffer to the PC
Send_Buffer2:
	CLR	bcount
	CLRH			; we are using X as an address pointer

	LDA	#$AA		; tell PC we are ready
	JSR	SENDCHAR

sb2f1:	JSR	GETCHAR
	TSTA
	BEQ	sb2fend

	LDX	bcount
	CLRH
	LDA	buffer,X
	JSR	SENDCHAR

	INC	bcount
	BRA	sb2f1

sb2fend: RTS
*==========================================================================
* Get the Filename from the PC
Get_Fname:
	CLR	bcount
	CLRH			; we are using X as an address pointer

gfnbr1: LDA	#$55		; we are ready for the next character
	JSR	SENDCHAR

	JSR	GETCHAR
	LDX	bcount
	STA	Fname,X

	BEQ	gfnbr2		; the last one

	INC	bcount
	BRA	gfnbr1

gfnbr2: RTS
*-----------------------------------------------------------------------------
* Get the File LBA
* We must have already found the Directory LBA and issued the Read LBA Command
Get_File:

* File Length of FILE to be found, length returned in flength
	JSR	File_Len

* Find the records
	MOV	#50,maxrecords

gfbr1:	JSR	Get_Dir_Record		; returns $FF if end of records
	BNE	gfbr2			; done, with no more records

* We have a file record
* Filename and LBA are in cbuffer [0..14] and [15..18]

* Convert the Filename to Upper case
*	JSR	UpperCase

* Compare the Filename with Requested Filename
	JSR	File_Comp

* If they match, then go to the next step, otherwise go again
	BEQ	gfbr3

	DEC	maxrecords       ; have we exceeded the max number of records?
	BNE	gfbr1 		 ; go again

* We have failed to find the file in the directory
gfbr2:	JSR	FLUSH		 ; flush any remaining data in the block 
	LDA	#$FF
	RTS

* We have a winner, File LBA is in LBA_Adr+n
gfbr3:

* Finally, instead of trying to calculate the number of bytes remaining 
*   we will just read and discard 2048 bytes
	JSR	FLUSH

	CLRA
	RTS
*==========================================================================
* File Length, length returned in flength
* string ends with '0'
File_Len:
        LDHX	#$0000
flbr1:  LDA	Fname,X
        BEQ	flbr2		; we are done
        INCX
        BRA	flbr1

flbr2:  DECX			; go back one
	STX	flength
        RTS
*==========================================================================
* Compare file names
* returns $FF if match, 0 if no match
File_Comp:
       	LDX	flength
	CLRH			; we are using X as an address pointer
fcbr1	LDA	Fname,X
	CMP	cbuffer,X
	BNE	fcbr2		; wrong
	DECX
	BPL	fcbr1

* we got here, so it is a match
	CLRA
	RTS

fcbr2:	LDA	#$FF
	RTS
*==========================================================================
* Convert to upper case, cbuffer, 14 characters
UpperCase:
	LDHX	#14
ucbr1:	LDA	cbuffer,X
	CMP	#$61
	BMI	ucbr2
	SUB	#$20
	STA	cbuffer,X
ucbr2:  DECX
	BPL	ucbr1
	RTS
*==========================================================================
* Use Our Filename
* Filename must be capital letters, terminated with 0.
FILENAME  FCB   'README.TXT',0
Our_Fname:
	LDHX	#15
ofnbr1:	LDA	FILENAME,X
	STA	Fname,X
	DECX
	BPL	ofnbr1
	RTS
*==========================================================================
* Get the Directory record
Get_Dir_Record:
* Start looking for the file
	CLRH				; we are using X as an address pointer

	JSR	READ_CD_WORD		; bytes 0,1
        MOV	byte_low,recordlength	; length of record (byte 0)
	BNE	gdr1			; a length of 0 means no more records

	LDA	#$08		
	RTS

* Get the File LBA, if this is not the File it will be ignored
gdr1:	JSR     READ_CD_WORD		; bytes 2,3

	MOV	byte_low,LBA_Adr+0
	MOV	byte_high,LBA_Adr+1

	JSR     READ_CD_WORD		; bytes 4,5

	MOV	byte_low,LBA_Adr+2
	MOV	byte_high,LBA_Adr+3

*   Fast Forward to byte 32 (file identifier length)  [word 16 - word 2]
	LDA	#14
	JSR	FFORWARD

*   byte_low is the file name length
*   byte_high is the first character in the file name
	LDA	byte_high
	STA	cbuffer+0	; byte 33 (word 17)

*   Get 14 characters
	MOV	#$01,bindex		; buffer index

	MOV	#07,wcount

gdr2:	  JSR     READ_CD_WORD

	  LDX	bindex
	  CLRH
	  LDA	byte_low
	  STA	cbuffer,X

	  INCX

	  LDA	byte_high
	  STA	cbuffer,X
	  INCX

	  STX	bindex

	  DEC	wcount
	  BNE	gdr2

* The file name is in cbuffer

* We are now at word 24, fast forward to the end of the record
	LDA	recordlength
	LSRA			; convert to words
	SUB	#24
	BEQ	gdr3
	BMI	gdr4		; less than or equal to 0
	JSR	FFORWARD

gdr3:	CLRA
	RTS

gdr4:	LDA	#$FE		; exceeded expected length of record
	RTS
*==========================================================================
******************************************************************************
* Various Test Functions for the CDROM Drive
******************************************************************************
*==========================================================================
* Test Program for Getting Table of Contents(10)
TEST_TOC:
* Hard Reset
*	JSR	Hard_Reset

* Start the Disc
	JSR	Start_Disc
	BEQ	toct1
	JMP	FLASH_ERR

* Wait_For_Ready
toct1:	JSR	Wait_For_Ready
	BEQ	toct2
	JMP	FLASH_ERR

toct2:	JSR	Tst_Read_TOC10

	RTS
*====================================================
TTOC10:	FCB   $00,$00		; 11,10
	FCB   $80,$00		; 9,8
	FCB   $20,$01		; 7,6
	FCB   $00,$00		; 5,4
	FCB   $00,$10		; 3,2
	FCB   $02,READ_TOC	; 1,0

* Read Table of Contents 10 (for audio CD tracks)
Tst_Read_TOC10:

* Form the command packet for Read TOC 10
	LDHX	#11			; X= 11->0 is packet[0..11]
ttc1:	   LDA	  TTOC10,X
	   STA	  buffer,X
	   DECX
	   BPL    ttc1

      JSR	Do_Packet		; returns $00 = no errors
      BEQ	ttc2
      JMP	FLASH_ERR

* wait while BSY
ttc2:

* get and decode the data
	JSR     READ_CD_WORD	; length

	MOV	byte_low,length+1	; length MSB
	MOV	byte_high,length+0	; length LSB

	LDA	length+0	; subtract 4+11
	ADD	#$F1
	STA	length+0

	LDA	length+1
	ADC	#$FF
	STA	length+1

	JSR     READ_CD_WORD	; first, last session (ignore)

* There are 11 bytes/record
ttclp:	JSR     READ_CD_WORD	; Session #, ADR:Control (ignore)

	JSR     READ_CD_WORD	; TNO (ignore), Point

	LDA	byte_high	; 0-99 is track number
	STA	track
	CMP	#99
	BMI	ttc3	

* not a track number 
	JSR     READ_CD_WORD	; byte 4, 5
	JSR     READ_CD_WORD	; byte 6, 7
	JSR     READ_CD_WORD	; byte 8, 9 
	JSR     READ_CD_WORD    ; byte 10, 0 (byte_high is byte 0 next record)
	BRA	ttc_odd

* we have a track number
ttc3:
	JSR     READ_CD_WORD	; byte 3, byte 4 (ignore)
	JSR     READ_CD_WORD	; byte 5, byte 6 (ignore)

	JSR     READ_CD_WORD	; M, S
	MOV	byte_low,Mval
	MOV	byte_high,Sval

	JSR     READ_CD_WORD	; F, first bute of next record
	MOV	byte_low,Fval

	JSR	Display_MSF

* byte_high is first byte of next record
ttc_odd:
	JSR	Do_Length
	BPL	ttc4		; msb less than zero means we are done

* we are done, so just read and discard 2048 bytes
	JSR	FLUSH
	RTS

* byte_high is byte 0 of next record, ignore it
ttc4:	JSR     READ_CD_WORD    ; byte 1, 2 (ignore)

	JSR     READ_CD_WORD	; byte 3 Point, byte 4 (ignore)

	LDA	byte_low	; byte 3: 0-99 is track number
	STA	track
	CMP	#99
	BMI	ttc5	

* not a track number 
	JSR     READ_CD_WORD	; byte 5, byte 6
	JSR     READ_CD_WORD	; byte 7, byte 8
	JSR     READ_CD_WORD	; byte 9, byte 10
	JMP	ttclp		; we are at an even boundary again

ttc5:	JSR     READ_CD_WORD	; byte 5, byte 6 (ignore)

	JSR     READ_CD_WORD	; byte 7, byte 8 (Mvalue)
	MOV	byte_high,Mval

	JSR     READ_CD_WORD	; S, F
	MOV	byte_low,Sval

	MOV	byte_high,Fval

	JSR	Display_MSF

	JSR	Do_Length
	BMI	ttc6		; msb less than zero means we are done
	JMP	ttclp

*  we will just read and discard 2048 bytes
ttc6:	JSR	FLUSH
	RTS
*==========================================================================
Do_Length:
	LDA	length		; subtract 11
	ADD	#$F5
	STA	length
	LDA	length+1
	ADC	#$FF
	STA	length+1
	RTS
*---------------------------------------------------------------------------
* displayed in hex, not BCD
Display_MSF:
	LDA	track
        LDX     #$80    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	Mval
        LDX     #$84    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	Sval
        LDX     #$87    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	Fval
        LDX     #$8A    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

*	LDA	length
*	LDX     #$C0    	; set cursor location
* 	JSR     DISPLAY_BINARY  ; Display Binary from ACC

*	LDA	length+1
*	LDX     #$C4    	; set cursor location
*	JSR     DISPLAY_BINARY  ; Display Binary from ACC

	JSR	WAIT1s
	RTS
*==========================================================================
TEST_PLAYMSF:

* Start the Disc
	JSR	Start_Disc
	BEQ	tpm1
	JMP	FLASH_ERR

* Wait_For_Ready
tpm1:	JSR	Wait_For_Ready
	BEQ	tpm2
	JMP	FLASH_ERR

tpm2:	JSR	Test_Play_MSF

	JSR	Get_Alt_Status
	LDX	#$D0
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	JSR	WAIT250ms

	RTS
*====================================================
* 	       M   F    S   (BCD)
PMSF:	FCB   00,02,32	; track 1
	FCB   01,32,45	; track 2
	FCB   03,10,45	; track 3
	FCB   37,07,45	; track 4
	FCB   38,41,45	; track 5

EM	EQU	99	; End M
ES	EQU	59	; End S
EF	EQU	74	; End F

* Play Audio MSF from Table
Test_Play_MSF:

* Form the command packet for PLAY_AUDIO_MSF
	LDX	#$0C		; track number-1
	STX	track
	CLRH			; we are using X as an address pointer

        MOV	#PLAY_AUDIO_MSF,buffer+11	; byte 0	Op Code
	CLR	buffer+10	; byte 1
	CLR	buffer+9	; byte 2

	LDA	PMSF,X
	STA	buffer+8	; byte 3	Start M
	STA	Mval

	INCX
	LDA	PMSF,X
	STA	buffer+7	; byte 4	Start S
	STA	Sval

	INCX
	LDA	PMSF,X
	STA	buffer+6	; byte 5	Start F
	STA	Fval

	MOV	#EM,buffer+5	; byte 6	Ending M
	MOV	#ES,buffer+4	; byte 7	Ending S
	MOV	#EF,buffer+3	; byte 8	Ending F
	CLR	buffer+2	; byte 9
	CLR	buffer+1	; byte 10
	CLR	buffer+0	; byte 11

	JSR	Do_Packet	; returns $00 = no errors
	BEQ	pm2
	JMP	FLASH_ERR

* done
pm2:	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

	JSR	Display_Buffer
	RTS
*---------------------------------------------------------------------------
* Display the data buffer
Display_Buffer:
	LDA	buffer+11
        LDX     #$80    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+10
        LDX     #$83    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+9
        LDX     #$86    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+8
        LDX     #$89    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+7
        LDX     #$8C    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+6
        LDX     #$8F    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+5
        LDX     #$92    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+4
        LDX     #$C0    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+3
        LDX     #$C3    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+2
        LDX     #$C6    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+1
        LDX     #$C9    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	buffer+01
        LDX     #$CC    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	RTS
*=============================================================================
******************************************************************************
*  Serial Command Mode Functions, called from PC
******************************************************************************
*==========================================================================
* Get the number of the function from the PC and go to it.
* Functions generally jump back to this function to get a new function.
CMD:    JSR     GETCHAR			; get a byte from the PC
        STA     char
        JSR     SENDCHAR		; echo the command

        LDA     char
        AND     #$1F			; limit commands to 32
        LSLA
        TAX
	CLRH

        LDA     (PTABLE),X		; address high
        STA     address

        LDA     (PTABLE+1),X		; address low
        STA     address+1

        LDHX	address
        JMP     ,X			; go there
*---------------------------------------------------------
PTABLE:   FDB   PROG0,PROG1,PROG2,PROG3,PROG4,PROG5,PROG6,PROG7,PROG8,PROG9
          FDB   PROG10,PROG11,PROG12,PROG13,PROG14,PROG15,PROG16,PROG17
          FDB   PROG18,PROG19,PROG20,PROG21,PROG22,PROG23,PROG24,PROG25
	  FDB	PROG26
*==========================================================================
* Send back H#AA to say we are alive *
PROG0:  JSR     WAIT1ms

	LDA     #$AA
        JSR     SENDCHAR
        JMP     CMD
*==========================================================================
* Receive Download
PROG1:  JSR     GETCHAR         ; # bytes to download
        STA     count

        JSR     GETCHAR         ; address high
        STA     address

        JSR     GETCHAR         ; address low
        STA     address+1

p1lp:   JSR     GETCHAR         ; character is in ACC
        LDHX	address
	STA	,X              ; STA from address

        LDA     address+1       ; increment the 16 bit pointer
        ADD     #$01
        STA     address+1

        LDA     address
        ADC     #$00
        STA     address

        DEC     count
        BNE     p1lp

        JMP     CMD
*==========================================================================
* Start the program
PROG2:  JMP     START_IT      ; Go to the User's Start Vector
*==========================================================================
* Send Version Number
PROG3:  JSR     WAIT1ms

	LDA     #$01
        JSR     SENDCHAR
        JSR     WAIT200us

        LDA     #$10
        JSR     SENDCHAR

        JMP     CMD
*==========================================================================
* Send Data Back
PROG4:  JSR     GETCHAR         ; # bytes to download
        STA     count

        JSR     GETCHAR         ; address high
        STA     address

        JSR     GETCHAR         ; address low
        STA     address+1

p4lp:   LDHX	address
	LDA	,X
        JSR     SENDCHAR        ; character is in ACC
        JSR     WAIT1ms

        LDA     address+1          ; increment the 16 bit pointer
        ADD     #$01
        STA     address+1

        LDA     address
        ADC     #$00
        STA     address

        DEC     count
        BNE     p4lp

        JMP     CMD
*==========================================================================
* Flash the LCD once
PROG5:	JSR	FLASH_LCD
	JMP     CMD
*==========================================================================
* Read Alt_Status and send it back
*    d7    d6   d5    d4    d3  d2  d1  d0
*   /Res  /RD  /WR  /CS1  /CS0  A2  A1  A0
PROG6:  JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	JSR	WAIT1ms
	LDA	status
	JSR	SENDCHAR
	JMP     CMD
*==========================================================================
* CDROM hardware-reset
PROG7:	MOV	#$FF,DDRA	; Port A Output

	MOV	#$F7,PORTA
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; Reset Low

	JSR	WAIT100us

	MOV	#$FF,PORTA
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD	; Reset high

	CLR	DDRA		; Port A Input

	JMP     CMD
*==========================================================================
* Get and Display the Ports
PROG8: 	JSR	WAIT10ms	; for compatibility with old system

	LDA	#$00
	JSR	SENDCHAR

* Get a single data and send it back (16 bit)
	JSR	DoData1		

* Now get the other registers (8 bit)
	LDA	#$01
	STA	PortAddress

P8lp:	JSR	ReadPort	; returns with port in ACC
	JSR	SENDCHAR

	INC	PortAddress
	LDA	#$08
	CMP	PortAddress
	BNE	P8lp

	JMP     CMD
*==========================================================================
* Write a type 3 packet with no Data coming back
*    (Eject, Open, Start_Disc, Stop_Disc, Play_Audio)
* data[0..11] -> buffer[11..0]
PROG9:
* Get the packet from the PC, 6 words = 12 bytes, low byte first
	MOV	#11,pcount

p9lp:	JSR	GETCHAR

	LDX	pcount
	CLRH			; we are using X as an address pointer
	STA	buffer,X
	DECX
	STX	pcount
	BPL	p9lp

* Send Packet Command
	JSR	Packet_Command		; send Packet Command
	BEQ	p9br1

	LDA	#$01			; Busy never went low
	JSR	SENDCHAR

	JSR	GETCHAR

	LDA	status	
	JSR	SENDCHAR

	JMP     CMD

* Give DRQ a chance to go high
p9br1:	MOV	#$20,pcount
p9lp2:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRSET	3,status,p9br2		; DRQ must be high
	JSR	WAIT1ms
	DEC	pcount
	BNE	p9lp2

	LDA	#$02			; DRQ never went high
	JSR	SENDCHAR

	JSR	GETCHAR

	LDA	status	
	JSR	SENDCHAR

	JMP     CMD

* Send the Packet
p9br2:
	JSR	Write_DBuffer   	; returns status in ACC

	LDA	#$03			; mission accomplished
	JSR	SENDCHAR

	JSR	GETCHAR

	LDA	status	
	JSR	SENDCHAR

	JMP     CMD
*==========================================================================
* ATAPI_IDENTIFY
PROG10:	
*	CLR	temp

* Give Busy a chance to go low
	MOV	#$20,pcount
p10a:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRCLR	7,status,p10b		; Busy must be Low
	JSR	WAIT1ms
	DEC	pcount
	BNE	p10a

	LDA	#$01			; Busy never went low
	JSR	SENDCHAR

	JSR	GETCHAR			; wait for PC
	CMPA	#$01
	BEQ     p10t1

	JMP	FLASH_ERR

p10t1	LDA	status	
	JSR	SENDCHAR
	JMP     CMD			; done

* Write the command
p10b:	MOV	#ATAPI_IDENTIFY,cmdsave
	JSR	Write_Command		; from cmdsave

* read the real status register (resets INTRQ)
	MOV	#$07,PortAddress
	JSR	ReadPort
	STA	status

* Give DRQ a chance to go high
	MOV	#$20,pcount
p10c:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRSET	3,status,p10d		; DRQ must be high
	JSR	WAIT1ms
	DEC	pcount
	BNE	p10c

	LDA	#$02			; DRQ never went high
	JSR	SENDCHAR

	JSR	GETCHAR			; wait for PC
	CMPA	#$01
	BEQ     p10t2

	JMP	FLASH_ERR

p10t2	LDA	status	
	JSR	SENDCHAR

	JMP     CMD			; done

p10d:	LDA	#$03			; ready to send data
	JSR	SENDCHAR

	JSR	GETCHAR			; wait for PC
	CMPA	#$01
	BEQ     p10t3

	JMP	FLASH_ERR

p10t3	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	JSR	SENDCHAR

p10e:	JSR	GETCHAR			; wait for PC
	CMPA	#$55
	BNE	p10f			; no more

	JSR	READ_CD_WORD

	LDA	byte_low
	JSR	SENDCHAR

	JSR	GETCHAR			; wait for PC
	CMPA	#$66
	BNE	p10f			; no more (error)

	LDA	byte_high
	JSR	SENDCHAR

	BRA	p10e

* we are done
p10f:	LDA	status	
	JSR	SENDCHAR
	JMP     CMD			; done
*---------------------------------------------------------
	JSR	GETCHAR			; wait for PC
	JSR	READ_CD_WORD

	LDA	byte_low
	JSR	SENDCHAR

	JSR	GETCHAR			; wait for PC

	LDA	byte_high
	JSR	SENDCHAR

	BRA	p10e
*==========================================================================
PROG11:	JSR	Get_Error_Rd	; Returns Error code in status as well as ACC
	JSR	SENDCHAR
	JMP     CMD
*==========================================================================
* Read Data
PROG12:	

*	CLR	temp

* Give Busy a chance to go low
	MOV	#$20,pcount
p12a:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRCLR	7,status,p12b		; Busy must be Low
	JSR	WAIT1ms
	DEC	pcount
	BNE	p12a

	LDA	#$01			; Busy never went low
	JSR	SENDCHAR

	JSR	GETCHAR			; wait for PC
	CMPA	#$01
	BEQ     p12t1

	JMP	FLASH_ERR

p12t1	LDA	status	
	JSR	SENDCHAR
	JMP     CMD			; done

p12b:
* Give DRQ a chance to go high
	MOV	#$20,pcount
p12c:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRSET	3,status,p12d		; DRQ must be high
	JSR	WAIT1ms
	DEC	pcount
	BNE	p12c

	LDA	#$02			; DRQ never went high
	JSR	SENDCHAR

	JSR	GETCHAR			; wait for PC
	CMPA	#$01
	BEQ     p12t2

	JMP	FLASH_ERR

p12t2	LDA	status	
	JSR	SENDCHAR

	JMP     CMD			; done

p12d:	LDA	#$03			; ready to send data
	JSR	SENDCHAR

	JSR	GETCHAR			; wait for PC
	CMPA	#$01
	BEQ     p12t3

	JMP	FLASH_ERR

p12t3	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	JSR	SENDCHAR

p12e:	JSR	GETCHAR			; wait for PC
	CMPA	#$55
	BNE	p12f			; no more

	JSR	READ_CD_WORD

	LDA	byte_low
	JSR	SENDCHAR

	JSR	GETCHAR			; wait for PC
	CMPA	#$66
	BNE	p12f			; no more (error)

	LDA	byte_high
	JSR	SENDCHAR

	BRA	p12e

* we are done
p12f:	LDA	status	
	JSR	SENDCHAR
	JMP     CMD			; done
*==========================================================================
* Write byte count low, byte count high, device select
PROG13:	JSR	WAIT1ms
	LDA	#$01
	JSR	SENDCHAR	; we are ready
*-----
	JSR	GETCHAR		; byte count low
	STA	datasave
	LDA	#BYTE_COUNT_LOW
	JSR	WritePort

	LDA	#$01
	JSR	SENDCHAR
*-----
	JSR	GETCHAR		; byte count high
	STA	datasave
	LDA	#BYTE_COUNT_HIGH
	JSR	WritePort

	LDA	#$01
	JSR	SENDCHAR
*-----
	JSR	GETCHAR		; device select
	STA	datasave
	LDA	#DRIVE_SEL
	JSR	WritePort

	LDA	#$02		; we are done
	JSR	SENDCHAR

	JMP	CMD
*==========================================================================
* Test SPI (maybe later)
PROG14:	JMP	CMD
*==========================================================================
FILE_LBA:   FCB   $10,$00,$00,$00	; least to most - Primary Descriptor
*--------------------------------
* Development Test: 
*    Get the Primary File Descriptor,
*    Get the Directory LBA, 
*    Send back the Primary Descriptor Volume
PROG15:	
* Hard Reset
*	JSR	Hard_Reset

* Start the Disc
	JSR	Start_Disc
	BEQ	p15br1
	ORA	#$10
	JSR	SENDCHAR
	JMP	CMD
p15br1:	

* Wait_For_Ready
	JSR	Wait_For_Ready
	BEQ	p15br2
	JSR	SENDCHAR	; $FF = timed out waiting for Device Ready
	JMP	CMD

* Initialize Buffer Pointers
p15br2:	JSR	Init_BPtrs

* LBA Address of Primary Descriptor
	LDHX	#$0003
p15br3:	LDA	FILE_LBA,X
	STA	LBA_Adr,X
	DECX
	BPL	p15br3

* Issue the Read_LBA Command
	JSR	Wait_For_Ready
	JSR	CMD_READ_LBA
	BEQ	p15br4
	ORA	#$20
	JSR	SENDCHAR
	JMP	CMD
p15br4:	

	LDA	#$00
	JSR	SENDCHAR		; tell PC to expect LBA addresses

* Send back the data
	JSR	Report_2kData

* We are done
	JMP	CMD
*==========================================================================
* Development Test: 
*    Get the Primary File Descriptor,
*    Get the Directory LBA, 
*    Send back the Directory
PROG16:	
* Hard Reset
*	JSR	Hard_Reset

* Test Error Messages
*	LDA	#$23		;$FF,$11,$12,$13,$21,$22,$23
*	JSR	SENDCHAR
*	JMP	CMD

* Start the Disc
	JSR	Start_Disc
	BEQ	p16br1
	ORA	#$10
	JSR	SENDCHAR
	JMP	CMD
p16br1:	

* Wait_For_Ready
	JSR	Wait_For_Ready
	BEQ	p16br2
	JSR	SENDCHAR	; $FF = timed out waiting for Device Ready
	JMP	CMD

* Initialize Buffer Pointers
p16br2:	JSR	Init_BPtrs

* LBA Address of Primary Descriptor
	LDHX	#$0003
p16br3:	LDA	FILE_LBA,X
	STA	LBA_Adr,X
	DECX
	BPL	p16br3

* Issue the Read_LBA Command
	JSR	Wait_For_Ready
	JSR	CMD_READ_LBA
	BEQ	p16br4
	ORA	#$20
	JSR	SENDCHAR
	JMP	CMD
p16br4:	

* Get the Directory LBA and tell it to the PC
	LDA	#$00
	JSR	SENDCHAR		; tell PC to expect LBA addresses

	JSR	Get_Dir_LBA
	LDA	#$80
        JSR     CLR_DISP
	JSR	Show_LBA

	LDA	LBA_Adr+0
	JSR	SENDCHAR

	JSR	GETCHAR
	LDA	LBA_Adr+1
	JSR	SENDCHAR

	JSR	GETCHAR
	LDA	LBA_Adr+2
	JSR	SENDCHAR

	JSR	GETCHAR
	LDA	LBA_Adr+3
	JSR	SENDCHAR

	JSR	GETCHAR

* The Directory LBA is set
* Issue the Read_LBA Command
	JSR	Wait_For_Ready
	JSR	CMD_READ_LBA
	BEQ	p16br5
	ORA	#$30
	JSR	SENDCHAR
	JMP	CMD
p16br5:	

* Send back the data
	JSR	SENDCHAR		; tell PC to expect data
	JSR	Wait_For_DRQ
	JSR	Report_2kData

* We are done
	JMP	CMD
*==========================================================================
* Development Test: 
*    Get the Primary File Descriptor,
*    Get the Directory LBA, 
*    Read the Directory, 
*    Send back Filenames and LBAs
PROG17:	
* Hard Reset
*	JSR	Hard_Reset

* Start the Disc
	JSR	Start_Disc
	BEQ	p17br1
	ORA	#$10
	JSR	SENDCHAR
	JMP	CMD
p17br1:	

* Wait_For_Ready
	JSR	Wait_For_Ready
	BEQ	p17br2
	JSR	SENDCHAR	; $FF = timed out waiting for Device Ready
	JMP	CMD

* Initialize Buffer Pointers
p17br2:	JSR	Init_BPtrs

* LBA Address of Primary Descriptor
	LDHX	#$0003
p17br3:	LDA	FILE_LBA,X
	STA	LBA_Adr,X
	DECX
	BPL	p17br3

* Issue the Read_LBA Command
	JSR	Wait_For_Ready
	JSR	CMD_READ_LBA
	BEQ	p17br4
	ORA	#$20
	JSR	SENDCHAR
	JMP	CMD
p17br4:	

* Get the Directory LBA
	JSR	Get_Dir_LBA
	LDA	#$80
        JSR     CLR_DISP
	JSR	Show_LBA

* The Directory LBA is set
* Issue the Read_LBA Command
	JSR	Wait_For_Ready
	JSR	CMD_READ_LBA
	BEQ	p17br5
	ORA	#$30
	JSR	SENDCHAR		; error
	JMP	CMD
p17br5:	

*	JMP	TST1			; Test this

	LDA	#$00		; tell PC we have gotten here
	JSR	SENDCHAR

	JSR	Wait_For_DRQ

* Find the records
	MOV	#50,maxrecords

p17br6:	JSR	Get_Dir_Record		; returns $FF if end of records
	BEQ	p17br7		

* We are done.
* Instead of trying to calculate the number of bytes remaining,
*    we will just read and discard 2048 bytes
	JSR	FLUSH

	LDA	#$55			; end of records
	JSR	SENDCHAR
	JMP	CMD

p17br7:	

* Send back the data
	LDA	LBA_Adr+0
	STA	cbuffer+15

	LDA	LBA_Adr+1
	STA	cbuffer+16

	LDA	LBA_Adr+2
	STA	cbuffer+17

	LDA	LBA_Adr+3
	STA	cbuffer+18

	LDA	#$00		; tell PC we have gotten here
*	JSR	SENDCHAR

	JSR	Send_Buffer

	DEC	maxrecords       ; have we exceeded the max number of records?
	BNE	p17br6		 ; go again

* We are done
	JMP	CMD
*==========================================================================
* Development Test: 
*    Get the Primary File Descriptor,
*    Get the Directory LBA, 
*    Read the Directory, 
*    Find the File LBA,
*    Send back the File
PROG18:	
	JSR	Get_Fname

*	JSR	Our_Fname		; ignore PC's Filename, use ours

* Hard Reset
*	JSR	Hard_Reset

* Start the Disc
	JSR	Start_Disc
	BEQ	p18br1
	ORA	#$10		; identify where the error came from
	JSR	SENDCHAR
	JMP	CMD
p18br1:	

* Wait_For_Ready
	JSR	Wait_For_Ready
	BEQ	p18br2
	JSR	SENDCHAR	; $FF = timed out waiting for Device Ready
	JMP	CMD

* Initialize Buffer Pointers
p18br2:	JSR	Init_BPtrs

* LBA Address of Primary Descriptor
	LDHX	#$0003
p18br3:	LDA	FILE_LBA,X
	STA	LBA_Adr,X
	DECX
	BPL	p18br3

* Issue the Read_LBA Command
	JSR	CMD_READ_LBA
	BEQ	p18br4
	ORA	#$20
	JSR	SENDCHAR
	JMP	CMD
p18br4:	

* Get the Directory LBA and tell it to the PC
	LDA	#$00
	JSR	SENDCHAR		; tell PC to expect LBA addresses

	JSR	Get_Dir_LBA

	LDA	LBA_Adr+0
	JSR	SENDCHAR

	JSR	GETCHAR
	LDA	LBA_Adr+1
	JSR	SENDCHAR

	JSR	GETCHAR
	LDA	LBA_Adr+2
	JSR	SENDCHAR

	JSR	GETCHAR
	LDA	LBA_Adr+3
	JSR	SENDCHAR

	JSR	GETCHAR

* The Directory LBA is set
* Issue the Read_LBA Command
	JSR	Wait_For_Ready
	JSR	CMD_READ_LBA
	BEQ	p18br5
	ORA	#$30
	JSR	SENDCHAR
	JMP	CMD
p18br5:	

* Send back the data
*	JSR	SENDCHAR		; tell PC to expect data
*	JSR	Report_2kData

* Find the file and report the data
	JSR	Get_File
	BEQ	p18br6
	LDA	#$F0			; did not find file
	JSR	SENDCHAR
	JMP	CMD
p18br6:

* The File LBA is set
* Issue the Read_LBA Command
	JSR	Wait_For_Ready
	JSR	CMD_READ_LBA
	BEQ	p18br7
	ORA	#$40
	JSR	SENDCHAR
	JMP	CMD
p18br7:	

* Send back the data
	JSR	SENDCHAR		; tell PC to expect data
	JSR	Report_2kData

* We are done
	JMP	CMD
*==========================================================================
*==========================================================================
* Make the TOC Table
* Play track number from PC
PROG19:
* Software Reset
	JSR	Soft_Reset

* Start the Disc
	JSR	Start_Disc
	BEQ	p19br1
	JMP	FLASH_LCD

* Wait_For_Ready
p19br1:	JSR	Wait_For_Ready
	BEQ	p19br2
	JMP	FLASH_LCD

p19br2:	JSR	Make_TOC10

	LDA	track_count
	JSR	SENDCHAR		; tell PC to expect data

* Send the data back
	CLR	track_ptr

p19br3:	JSR	GETCHAR
	CMPA	#$00
	BNE	p19br4

	LDX	track_ptr
	CLRH
	LDA	CBUFFER,X
	JSR	SENDCHAR

	INC	track_ptr
	BRA	p19br3

* Get the track number and play it
p19br4: JSR	GETCHAR
	CMP	#$00
	BEQ	p19br5

* Play the track
	STA	track_save
	JSR	Report_MSF

	JSR	Wait_For_Ready
	LDA	track_save
	JSR	Play_Track
	BRA	p19br4		; go again

* We are done
p19br5:	JMP	CMD
*==========================================================================
* Get and send back drive capabilities
PROG20:	

* Use our own packet
	JSR	Get_Caps

* Send Packet Command
	JSR	Packet_Command		; send Packet Command
	BEQ	p20br1

	LDA	#$01			; Busy never went low
	JSR	SENDCHAR
	JMP     CMD

* Give DRQ a chance to go high
p20br1:	MOV	#$40,pcount
p20lp2:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRSET	3,status,p20br2		; DRQ must be high
	JSR	WAIT1ms
	DEC	pcount
	BNE	p20lp2

	LDA	#$02			; DRQ never went high
	JSR	SENDCHAR
	JMP     CMD

* Send the Packet
p20br2:	JSR	Write_DBuffer   	; returns status in ACC

* Give Busy a chance to go low
	MOV	#$40,pcount
p20ct1:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRCLR	7,status,p20ct2		; Busy must be Low
	JSR	WAIT1ms
	DEC	pcount
	BNE	p20ct1
z	LDA	#$03			; Busy never went low
	JSR	SENDCHAR
	JMP     CMD

p20ct2:	

* Give DRQ a chance to go high
	MOV	#$40,pcount
p20ct3:	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	BRSET	3,status,p20ct4		; DRQ must be high
	JSR	WAIT1ms
	DEC	pcount
	BNE	p20ct3

	LDA	#$04		; DRQ did not go high
	JSR	SENDCHAR
	JMP     CMD

p20ct4:
	LDA	#$05		; success
	JSR	SENDCHAR

* Show me the data
p20ct5:	JSR	GCDATA
	BEQ	p20ct6

	LDA	#$06		; bad news, no data
	JSR	SENDCHAR

p20ct6: LDA	#20		; tell it we are ready to send 20 bytes
	JSR	SENDCHAR	

	CLR	vcount

p20ct7:	JSR	GETCHAR			; wait for PC

	LDX	vcount
	CLRH
	LDA	CBUFFER,X
	JSR	SENDCHAR
	
	LDA	vcount
	INCA
	STA	vcount
	CMPA	#20

	BNE	p20ct7

* we are done	
	JMP	CMD
*==========================================================================
* Get Subcode
PROG21: JSR	Read_SubCode	; returns 16 bytes in buffer

	JSR	Send_Buffer2	; buffer (instead of cbuffer)

	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
	JSR	SENDCHAR

	JMP	CMD
*==========================================================================
* Play test, send Subcode back until we are told to stop
PROG22: CLR	wait_status

p22lp:	JSR	Read_SubCode	; returns 16 bytes in buffer

	JSR	Send_Buffer2	; sends back $AA, expects non-zero data,
* 				  sends buffer data

	JSR	Get_Alt_Status	; Returns Alt_Status in status as well as ACC
*	JSR	SENDCHAR	; normally we would send it here

* (if status(d0) = '0' it's ok, otherwise the door is probably open)
*   if wait_status(d0) = '0' and status(d0) = 0: continue
*   if wait_status(d0) = '0' and status(d0) = 1: set wait_status, continue
*   if wait_status(d0) = '1' and status(d0) = 0: reset wait_status, restart
*   if wait_status(d0) = '1' and status(d0) = 1: continue
*
* This can be simplified as:
*  if wait_status(d0) = '1' and status(d0) = 0: 
*		copy status(d0) to wait_status(d0), restart
*  else:        copy status(d0) to wait_status(d0), continue
	LDA	wait_status
	EOR	status
	AND	#$01
	BEQ	p22br1

	LDA	status
	AND	#$01
	STA	wait_status

* restart
	JSR	PLAY_Disc
	BRA	p22br2

p22br1:	LDA	status
	AND	#$01
	STA	wait_status

p22br2:	LDA	status
	JSR	SENDCHAR	; instead we will send it here

	JSR	GETCHAR
	TSTA
	BNE	p22lp

	JMP	CMD
*==========================================================================
* Development Test: Get the LBA block and send it back
PROG23:	

* Get LBA from the PC
	CLR	pcount

p23br1:	CLRA
	JSR	SENDCHAR

	JSR	GETCHAR
	LDX	pcount
	CLRH
	STA	LBA_Adr,x
	
	INC	pcount
	LDA	pcount
	CMP	#4
	BNE	p23br1

* Start the Disc
	JSR	Start_Disc
	BEQ	p23br2
	ORA	#$10
	JSR	SENDCHAR
	JMP	CMD
p23br2:	

* Wait_For_Ready
	JSR	Wait_For_Ready
	BEQ	p23br3
	JSR	SENDCHAR	; $FF = timed out waiting for Device Ready
	JMP	CMD

* Initialize Buffer Pointers
p23br3:	JSR	Init_BPtrs

* We already have the LBA values

* Issue the Read_LBA Command
	JSR	Wait_For_Ready
	JSR	CMD_READ_LBA
	BEQ	p23br4
	ORA	#$20
	JSR	SENDCHAR
	JMP	CMD
p23br4:	

	LDA	#$00
	JSR	SENDCHAR		; tell PC to expect LBA addresses

* Send back the data
	JSR	Report_2kData

* We are done
	JMP	CMD
*==========================================================================
* Software Reset
PROG24:	
	JSR	Soft_Reset
	JMP	CMD
*==========================================================================
* Debugging #1
PROG25:	JSR	$0100
	JMP	CMD
*==========================================================================
* Debugging #2
PROG26:	JSR	$0140
	JMP	CMD
*==========================================================================
*==========================================================================
PLAY_Disc:

* Make Command Packet for 'Play Audio MSF'
	LDHX	#11			; X= 11->0 is packet[0..11]
mpl1:	   LDA	  PLMSF,X
	   STA	  buffer,X
	   DECX
	   BPL    mpl1

* Send Packet Command
       JSR	Do_Packet

       RTS

PLMSF:	FCB   $00,$00		  ; 11,10
	FCB   $00,$0c		  ; 9,8   
	FCB   $0b,$49		  ; 7,6   
	FCB   $02,$01		  ; 5,4   
	FCB   $00,$00		  ; 3,2   
	FCB   $00,PLAY_AUDIO_MSF  ; 1,0   
*===========================================================================
Report_MSF:
* Get the MSF data from the TOC table
* Track number in ACC
	DECA			; track 1 = ptr 0
	LDX	#3		; 3 entries/track
	MUL			; low 8 bits in ACC
	TAX
	STX	address

* Get MSF
	LDX	address
	CLRH
	LDA	CBUFFER,X
	STA	Mval

	INCX
	LDA	CBUFFER,X
	STA	Sval

	INCX
	LDA	CBUFFER,X
	STA	Fval

* send back MSF
	JSR	GETCHAR
	LDA	address
	JSR	SENDCHAR

	JSR	GETCHAR
	LDA	Mval
	JSR	SENDCHAR

	JSR	GETCHAR
	LDA	Sval
	JSR	SENDCHAR

	JSR	GETCHAR
	LDA	Fval
	JSR	SENDCHAR

	RTS
* ====================================================
Show_LBA:
	LDA	LBA_Adr+3
	LDX	#$80
	FCB	$8C		; CLRH
	JSR	DISPLAY_BINARY

	LDA	LBA_Adr+2
	LDX	#$83
	FCB	$8C		; CLRH
	JSR	DISPLAY_BINARY

	LDA	LBA_Adr+1
	LDX	#$86
	FCB	$8C		; CLRH
	JSR	DISPLAY_BINARY

	LDA	LBA_Adr+0
	LDX	#$89
	FCB	$8C		; CLRH
	JSR	DISPLAY_BINARY

	RTS
*==========================================================================
******************************************************************************
* Vector Redirection Functions
******************************************************************************
*==========================================================================
* So that we can still use the Vector Indirection Table for development,
*  Put the address of the Main Timer Overflow function into the Vector 
*  Indirection Table.
tirq_adr:   FDB	  TIRQ
INIT_TIRQ:
	LDA	tirq_adr
	STA	USER_T1OFVEC

	LDA	tirq_adr+1
	STA	USER_T1OFVEC+1

	CLR	irqflag

	RTS
*==========================================================================
* Timer 1 Overflow Interrupt (USER_T1_OF_VEC)
TIRQ:	PSHH		; save H Register

	JSR	DO_SWITCHES	; read and debounce

	JSR	SWTRANS

	MOV	#$FF,irqflag

	BCLR	TOF,T1SC	; reset interrupt flag

TIRQRET:
	PULH		; restore H Register
	RTI
*==========================================================================
******************************************************************************
* Vector Redirection Table Functions
******************************************************************************
*==========================================================================
* Put Vector routines here so the security code will always be the same
	ORG	$D000		
* Timebase 						($021C)
* Jump to Address in User's Vector Table
TIM_INT:      LDA       #$CC	
              STA       JMPINSTR

              LDA       USER_VEC+0	; ($021C)
              STA       address

              LDA       USER_VEC+1	; ($021D)
              STA       address+1

              JMP       JMPINSTR
*------------------------------------ *
* A/D conversion complete				($021E)
* Jump to Address in User's Vector Table
ADC_INT:      LDA       #$CC	
              STA       JMPINSTR

              LDA       USER_VEC+2	; ($021E)
              STA       address

              LDA       USER_VEC+3	; ($021F)
              STA       address+1

              JMP       JMPINSTR
*------------------------------------ *
* Keyboard Vector					($0220)
* Jump to Address in User's Vector Table
KB_INT:       LDA       #$CC	
              STA       JMPINSTR

              LDA       USER_VEC+4	; ($0220)
              STA       address

              LDA       USER_VEC+5	; ($0221)
              STA       address+1

              JMP       JMPINSTR
*------------------------------------ *
* SCI Transmit Vector					($0222)
* Jump to Address in User's Vector Table
SCI_TX_INT:   LDA       #$CC	
              STA       JMPINSTR

              LDA       USER_VEC+6	; ($0222)
              STA       address

              LDA       USER_VEC+7	; ($0223)
              STA       address+1

              JMP       JMPINSTR
*------------------------------------ *
* SCI Receive Vector					($0224)
* Jump to Address in User's Vector Table
SCI_RX_INT:   LDA       #$CC	
              STA       JMPINSTR

              LDA       USER_VEC+8	; ($0224)
              STA       address

              LDA       USER_VEC+9	; ($0225)
              STA       address+1

              JMP       JMPINSTR
*------------------------------------ *
* SCI Error Vector					($0226)
* Jump to Address in User's Vector Table
SCI_ERR_INT:  LDA       #$CC	
              STA       JMPINSTR

              LDA       USER_VEC+10	; ($0226)
              STA       address

              LDA       USER_VEC+11	; ($0227)
              STA       address+1

              JMP       JMPINSTR
*------------------------------------ *
* SPI Transmit Vector					($0228)
* Jump to Address in User's Vector Table
SPI_TX_INT:   LDA       #$CC	
              STA       JMPINSTR

              LDA       USER_VEC+12	; ($0228)
              STA       address

              LDA       USER_VEC+13	; ($0229)
              STA       address+1

              JMP       JMPINSTR
*------------------------------------ *
* SPI Receive Vector					($022A)
* Jump to Address in User's Vector Table
SPI_RX_INT:   LDA       #$CC	
              STA       JMPINSTR

              LDA       USER_VEC+14	; ($022A)
              STA       address

              LDA       USER_VEC+15	; ($022B)
              STA       address+1

              JMP       JMPINSTR
*------------------------------------ *
* Timer 2 Overflow Vector				($022C)
* Jump to Address in User's Vector Table
TIM2_OF_INT:  LDA       #$CC	
              STA       JMPINSTR

              LDA       USER_VEC+16	; ($022C)
              STA       address

              LDA       USER_VEC+17	; ($022D)
              STA       address+1

              JMP       JMPINSTR
*------------------------------------ *
* Timer 2 Channel 1 Vector				($022E)
* Jump to Address in User's Vector Table
TIM2_CH1_INT: LDA       #$CC	
              STA       JMPINSTR

              LDA       USER_VEC+18	; ($022E)
              STA       address

              LDA       USER_VEC+19	; ($022F)
              STA       address+1

              JMP       JMPINSTR
*------------------------------------ *
* Timer 2 Channel 0 Vector				($0230)
* Jump to Address in User's Vector Table
TIM2_CH0_INT: LDA       #$CC	
              STA       JMPINSTR

              LDA       USER_VEC+20	; ($0230)
              STA       address

              LDA       USER_VEC+21	; ($0231)
              STA       address+1

              JMP       JMPINSTR
*------------------------------------ *
* Timer 1 Overflow Vector				($0232)
* Jump to Address in User's Vector Table
TIM1_OF_INT:  LDA       #$CC	
              STA       JMPINSTR

              LDA       USER_VEC+22	; ($0232)
              STA       address

              LDA       USER_VEC+23	; ($0233)
              STA       address+1

              JMP       JMPINSTR
*------------------------------------ *
* Timer 1 Channel 1 Vector				($0234)
* Jump to Address in User's Vector Table
TIM1_CH1_INT: LDA       #$CC	
              STA       JMPINSTR

              LDA       USER_VEC+24	; ($0234)
              STA       address

              LDA       USER_VEC+25	; ($0235)
              STA       address+1

              JMP       JMPINSTR
*------------------------------------ *
* Timer 1 Channel 0 Vector				($0236)
* Jump to Address in User's Vector Table
TIM1_CH0_INT: LDA       #$CC	
              STA       JMPINSTR

              LDA       USER_VEC+26	; ($0236)
              STA       address

              LDA       USER_VEC+27	; ($0237)
              STA       address+1

              JMP       JMPINSTR
*------------------------------------ *
* PLL Vector						($0238)
* Jump to Address in User's Vector Table
PLL_INT:      LDA       #$CC	
              STA       JMPINSTR

              LDA       USER_VEC+28	; ($0238)
              STA       address

              LDA       USER_VEC+29	; ($0239)
              STA       address+1

              JMP       JMPINSTR
*------------------------------------ *
* IRQ Vector						($023A)
* Jump to Address in User's Vector Table
IRQ_INT:      LDA       #$CC	
              STA       JMPINSTR

              LDA       USER_VEC+30	; ($023A)
              STA       address

              LDA       USER_VEC+31	; ($023B)
              STA       address+1

              JMP       JMPINSTR
*------------------------------------ *
* Software Interrupt					($023C)
* Jump to Address in User's Vector Table
SWI_INT:      LDA       #$CC	
              STA       JMPINSTR

              LDA       USER_VEC+32	; ($023C)
              STA       address

              LDA       USER_VEC+33	; ($023D)
              STA       address+1

              JMP       JMPINSTR
*------------------------------------ *
* Beginning of program on reset				($023E)
* Jump to Address in User's Vector Table
START_IT:     LDA       #$CC	
              STA       JMPINSTR

              LDA       USER_VEC+34	; ($023E)
              STA       address

              LDA       USER_VEC+35	; ($023F)
              STA       address+1

              JMP       JMPINSTR
*============================================================================
REAL_START:   JMP	START
*============================================================================
	FCB	$00,$11,$22,$33,$44,$55,$66,$77
	FCB	$88,$99,$AA,$BB,$CC,$DD,$EE,$FF


