* gp20.h
*
* Standalone Controller for an IDE CD-ROM Drive
* Rev 1.1 
* Jed Margolin
*
* MC68HC908GP20 Equates
*
* I/O Ports
PORTA   EQU    $0000      ;I/O port A
PORTB   EQU    $0001      ;I/O port B
PORTC   EQU    $0002      ;I/O port C
PORTD   EQU    $0003      ;I/O port D
;
DDRA    EQU    $0004      ;Data direction for port A
DDRB    EQU    $0005      ;Data direction for port B
DDRC    EQU    $0006      ;Data direction for port C
DDRD    EQU    $0007      ;Data direction for port D
;
PORTE   EQU    $0008      ;I/O port D
DDRE    EQU    $000C      ;Data direction for port D
;
PAPUP	EQU	$000D
PCPUP	EQU	$000E
PDPUP	EQU	$000F
* Port B and Port E do not have pullup registers
*--------------------------------------------------------------------------
; SCI
SCC1	EQU	$0013	; SCI Control Register 1 
ENSCI   EQU	$6	; Enable SCI
;
SCC2	EQU	$0014	; SCI Control Register 2 
RE	EQU	$2	; Receive Enable
TE	EQU	$3	; Transmit Enable
;
SCC3	EQU	$0015	; SCI Control Register 3 
;
SCS1	EQU	$0016	; SCI Status Register 1
TC      EQU	$6	; Transmission complete
SCRF	EQU	$5	; SCI Receiver Full Bit
;
SCS2	EQU	$0017	; SCI Status Register 2
;
SCDR	EQU	$0018	; SCI Data Register
;
SCBR	EQU	$0019	; SCI Baud Rate Register
*--------------------------------------------------------------------------
; Configuration Registers
CONFIG2	EQU	$001E
CONFIG1	EQU	$001F
*--------------------------------------------------------------------------
; Timer Registers
T1SC	EQU	$0020	; Timer 1 Status and Control Register
TOF	EQU	7	;   Timer Overflow Flag (R)
TOIE	EQU     6	;   Timer Overflow Interrput Enable (RW) 
TSTOP	EQU	5	;   Timer Stop (RW)
TRST	EQU	4	;   Timer Reset (W)


T1CNTH	EQU	$0021	; Timer 1 Counter Register High
T1CNTL	EQU	$0022	; Timer 1 Counter Register Low

T1MODH	EQU	$0023	; Timer 1 Modulo Register High
T1MODL	EQU	$0024	; Timer 1 Modulo Register Low

T1SC0	EQU	$0025	; Timer 1 Channel 0 Status and Control Register
T1CH0H	EQU	$0026	; Timer 1 Channel 0 Register High
T1CH0L	EQU	$0027	; Timer 1 Channel 0 Register Low

T1SC1	EQU	$0028	; Timer 1 Channel 1 Status and Control Register
T1CH1H	EQU	$0029	; Timer 1 Channel 1 Register High
T1CH1L	EQU	$002A	; Timer 1 Channel 1 Register Low
;---------
T2SC	EQU	$002B	; Timer 2 Status and Control Register

T2CNTH	EQU	$002C	; Timer 2 Counter Register High
T2CNTL	EQU	$002D	; Timer 2 Counter Register Low

T2MODH	EQU	$002E	; Timer 2 Modulo Register High
T2MODL	EQU	$002F	; Timer 2 Modulo Register Low

T2SC0	EQU	$0030	; Timer 2 Channel 0 Status and Control Register
T2CH0H	EQU	$0031	; Timer 2 Channel 0 Register High
T2CH0L	EQU	$0032	; Timer 2 Channel 0 Register Low

T2SC1	EQU	$0033	; Timer 2 Channel 1 Status and Control Register
T2CH1H	EQU	$0034	; Timer 2 Channel 1 Register High
T2CH1L	EQU	$0035	; Timer 2 Channel 1 Register Low
*--------------------------------------------------------------------------
* PLL Registers
PCTL	EQU	$0036		  ; PLL Control -> P,E
PLLON	EQU	$5
BCS	EQU	$4
;
PBWC	EQU	$0037		  ; PLL Bandwidth Control (Auto)
PMSH    EQU	$0038  		  ; PLL Multiplier Select High byte
PMSL    EQU	$0039		  ; PLL Multiplier Select Low byte
PMRS	EQU	$003A		  ; PLL VCO Range Select ->L
PMDS	EQU	$003B		  ; PLL Reference Divider Select  ->R
*--------------------------------------------------------------------------
* System Registers 
SBSR		equ	$FE00	; 
SRSR		equ	$FE01	; 
SUBAR		equ	$FE02	; 
SBFCR		equ	$FE03	; 
INT1		equ	$FE04	; 
INT2		equ	$FE05	; 
INT3		equ	$FE06	; 
FLTCR		equ	$FE07	; 
FLCR		equ	$FE08	; 
BRKH		equ	$FE09	; 
BRKL		equ	$FE0A	; 
BRKSCR		equ	$FE0B	; 
LVISR		equ	$FE0C	; 
;
FLBPR		equ	$FF80	; 
*--------------------------------------------------------------------------
VECTORS		equ     $FFDC   ; Start of Reset/Interrupt vectors

RAMstart        equ     $0040   ; Start of on chip RAM ($40 - $23F) 512 bytes 

ROMstart        equ     $B000   ; Start of program in User Flash 
*				  $B000 - $FDFF (19,968 bytes)
*--------------------------------------------------------------------------
* For Pod Program
* RAM is $0040 - $023F  (512 bytes)  We will assign:
*   $0040 - $004F for Pod operation (16 bytes)
*   $0050 - $00DF for User RAM ($90 = 144 bytes)
*   $00E0 - $00FF for Stack ($20 = 32 bytes)
*   $0100 - $021B for User Emulation RAM ($11C = 284 bytes)
*   $021C - $023F for User Vectors ($24 = 36 bytes) 
USER_RAM        equ     $0050
USER_EMRAM      equ     $0100
USER_VEC	equ     $021C	  ; Use the same order as the Real Vectors
USER_T1OFVEC    equ     USER_VEC+22  ; so we can use it either way

* Note that the real vector table locations $FFF6-FFFD (8 bytes) are used 
*   for the security code. These vectors are:
*	T1_CH0_VEC  FDB   TIM1_CH0_INT    ; Timer 1 Channel 0 Vector
*	PLL_VEC	    FDB   PLL_INT         ; PLL Vector
*	IRQ_VEC     FDB   IRQ_INT         ; IRQ Vector
*	SWI_VEC     FDB   SWI_INT         ; Software Interrupt
*============================================================================
	ORG	VECTORS
TBASEVEC    FDB   TIM_INT         ; Timebase				
ADCVEC	    FDB   ADC_INT         ; A/D conversion complete		
KBVEC	    FDB   KB_INT          ; Keyboard Vector			
SCI_TX_VEC  FDB   SCI_TX_INT      ; SCI Transmit Vector			
SCI_RX_VEC  FDB   SCI_RX_INT      ; SCI Receive Vector			
SCI_ERR_VEC FDB   SCI_ERR_INT     ; SCI Error Vector			
SPI_TX_VEC  FDB   SPI_TX_INT      ; SPI Transmit Vector			
SPI_RX_VEC  FDB   SPI_RX_INT      ; SPI Receive Vector			
T2_OF_VEC   FDB   TIM2_OF_INT     ; Timer 2 Overflow Vector		
T2_CH1_VEC  FDB   TIM2_CH1_INT    ; Timer 2 Channel 1 Vector		
T2_CH0_VEC  FDB   TIM2_CH0_INT    ; Timer 2 Channel 0 Vector		
T1_OF_VEC   FDB   TIM1_OF_INT     ; Timer 1 Overflow Vector		
T1_CH1_VEC  FDB   TIM1_CH1_INT    ; Timer 1 Channel 1 Vector		
T1_CH0_VEC  FDB   TIM1_CH0_INT    ; Timer 1 Channel 0 Vector		
PLL_VEC	    FDB   PLL_INT         ; PLL Vector				
IRQ_VEC     FDB   IRQ_INT         ; IRQ Vector				
SWI_VEC     FDB   SWI_INT         ; Software Interrupt			
RESET_VEC   FDB   REAL_START      ; Beginning of program on reset	
*============================================================================
* GP20 Pod Hardware Equates
TX      EQU     0	; Port E
RX      EQU     1	; Port E
;
nCS0    EQU     0	; Port C
nCS1    EQU     1	; Port C
nWR     EQU     2	; Port C
nRD     EQU     3	; Port C
SWEN    EQU     4	; Port C
;
PB6	EQU	6	; read from Port A (aux)
PB5	EQU	5	; read from Port A (aux)
PB4	EQU	4	; read from Port A (aux)
PB3	EQU	3	; read from Port A (aux)
PB2	EQU	2	; read from Port A (aux)
PB1	EQU	1	; read from Port A (aux)
PB0	EQU	0	; read from Port A (aux)
;
LATCH	EQU	4	; Port D
LCDEN	EQU	5	; Port D
LCDRS	EQU	6	; Latch Port A	lcd Register Select
LCDRW	EQU	7	; Latch Port A	lcd R/W, 0 = write
;
* PB0 - PB7  ->  LCD0 - LCD7
*============================================================================
* CDROM Equates
CS0	EQU	0	; /CS0	  Port C
CS1	EQU	1	; /CS1	  Port C
WR	EQU	2	; /WR	  Port C
RD	EQU	3	; /RD	  Port C

ATAPI_NOP		EQU	$00
ATAPI_PACKET_CMD	EQU	$A0
ATAPI_IDENTIFY		EQU	$A1
SERVICE			EQU	$A2
SOFT_RESET_CMD		EQU	$08
START_STOP_EJECT	EQU	$1B
INQUIRY			EQU	$12
PLAY_AUDIO_MSF		EQU	$47
PLAY_AUDIO_LBA		EQU	$45
STOP			EQU	$4E
READ_DATA10		EQU	$28
READ_DATA12		EQU	$A8
TEST_UR			EQU	$00
READ_TOC		EQU	$43
MODE_SENSE              EQU	$5A
SUBCHAN			EQU	$42

BYTE_COUNT_LOW		EQU	$04
BYTE_COUNT_HIGH		EQU	$05
DRIVE_SEL		EQU	$06
*============================================================================
